{-# LANGUAGE MagicHash #-}
{-# OPTIONS_GHC -fno-warn-overlapping-patterns #-}

module Curry_AbstractCurry
  ( C_MName, C_QName, C_CTVarIName, C_CField, C_Arity, C_CVarIName
  , C_CVisibility (..), C_CurryProg (..), C_CTypeDecl (..), C_CConsDecl (..)
  , C_CFieldDecl (..), C_CTypeExpr (..), C_COpDecl (..), C_CFixity (..)
  , C_CFuncDecl (..), C_CRule (..), C_CRhs (..), C_CLocalDecl (..)
  , C_CPattern (..), C_CExpr (..), C_CLiteral (..), C_CStatement (..)
  , C_CCaseType (..), d_C_version, d_C_readCurry, d_C_readUntypedCurry
  , d_C_readCurryWithParseOptions, d_C_readUntypedCurryWithParseOptions
  , d_C_abstractCurryFileName, d_C_untypedAbstractCurryFileName
  , d_C_readAbstractCurryFile, d_C_tryReadACYFile, d_C_writeAbstractCurryFile )
  where

import Basics
import qualified Curry_Char
import qualified Curry_Directory
import qualified Curry_Distribution
import qualified Curry_FilePath
import qualified Curry_Maybe
import qualified Curry_Prelude
import qualified Curry_ReadShowTerm


type C_MName = Curry_Prelude.OP_List Curry_Prelude.C_Char

type C_QName = Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
  Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char)

type C_CTVarIName = Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int
  (Curry_Prelude.OP_List Curry_Prelude.C_Char)

type C_CField t0 = Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_Tuple2
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List
  Curry_Prelude.C_Char)) t0

type C_Arity = Curry_Prelude.C_Int

type C_CVarIName = Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int
  (Curry_Prelude.OP_List Curry_Prelude.C_Char)

data C_CVisibility
  = C_Public
  | C_Private
  | Choice_C_CVisibility Cover ID C_CVisibility C_CVisibility
  | Choices_C_CVisibility Cover ID [C_CVisibility]
  | Fail_C_CVisibility Cover FailInfo
  | Guard_C_CVisibility Cover Constraints C_CVisibility

instance Show C_CVisibility where
  showsPrec d (Choice_C_CVisibility cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CVisibility cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CVisibility cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CVisibility cd info) = showChar '!'
  showsPrec _ C_Public = showString "Public"
  showsPrec _ C_Private = showString "Private"

instance Read C_CVisibility where
  readsPrec _ s = readParen False (\r -> [(C_Public, r0) | (_, r0) <-
    readQualified "AbstractCurry" "Public" r]) s ++ readParen False (\r ->
    [(C_Private, r0) | (_, r0) <- readQualified "AbstractCurry" "Private" r]) s

instance NonDet C_CVisibility where
  choiceCons = Choice_C_CVisibility
  choicesCons = Choices_C_CVisibility
  failCons = Fail_C_CVisibility
  guardCons = Guard_C_CVisibility
  try (Choice_C_CVisibility cd i x y) = tryChoice cd i x y
  try (Choices_C_CVisibility cd i xs) = tryChoices cd i xs
  try (Fail_C_CVisibility cd info) = Fail cd info
  try (Guard_C_CVisibility cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CVisibility cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CVisibility cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CVisibility cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CVisibility cd i _) = error
    ("AbstractCurry.CVisibility.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CVisibility cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CVisibility cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CVisibility where
  generate s c = Choices_C_CVisibility c (freeID [0, 0] s) [C_Public, C_Private]

instance NormalForm C_CVisibility where
  ($!!) cont C_Public d cs = cont C_Public d cs
  ($!!) cont C_Private d cs = cont C_Private d cs
  ($!!) cont (Choice_C_CVisibility cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CVisibility cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CVisibility cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CVisibility cd info) _ _ = failCons cd info
  ($##) cont C_Public d cs = cont C_Public d cs
  ($##) cont C_Private d cs = cont C_Private d cs
  ($##) cont (Choice_C_CVisibility cd i x y) d cs = gnfChoice cont cd i x y cd
    cs
  ($##) cont (Choices_C_CVisibility cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CVisibility cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CVisibility cd info) _ _ = failCons cd info
  showCons C_Public = "AbstractCurry.Public"
  showCons C_Private = "AbstractCurry.Private"
  showCons x = error ("AbstractCurry.CVisibility.showCons: no constructor: " ++
    show x)
  searchNF _ cont C_Public = cont C_Public
  searchNF _ cont C_Private = cont C_Private
  searchNF _ _ x = error
    ("AbstractCurry.CVisibility.searchNF: no constructor: " ++ show x)

instance Unifiable C_CVisibility where
  (=.=) C_Public C_Public d cs = C_Success
  (=.=) C_Private C_Private d cs = C_Success
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) C_Public C_Public d cs = C_Success
  (=.<=) C_Private C_Private d cs = C_Success
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i C_Public = (i :=: ChooseN 0 0) : concat []
  bind cd i C_Private = (i :=: ChooseN 1 0) : concat []
  bind d i (Choice_C_CVisibility cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CVisibility cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j
    xs
  bind d i (Choices_C_CVisibility cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CVisibility cd i _) = error
    ("AbstractCurry.CVisibility.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CVisibility cd info) = [Unsolvable info]
  bind d i (Guard_C_CVisibility cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i C_Public = (i :=: ChooseN 0 0) : []
  lazyBind cd i C_Private = (i :=: ChooseN 1 0) : []
  lazyBind d i (Choice_C_CVisibility cd j x y) = [ConstraintChoice cd j
    (lazyBind d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CVisibility cd j@(FreeID _ _) xs) = lazyBindOrNarrow d
    i cd j xs
  lazyBind d i (Choices_C_CVisibility cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CVisibility cd i _) = error
    ("AbstractCurry.CVisibility.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CVisibility cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CVisibility cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CVisibility where
  (=?=) (Choice_C_CVisibility cd i x y) z d cs = narrow cd i
    ((Curry_Prelude.=?=) x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CVisibility cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CVisibility cd c e) y d cs = guardCons cd c
    ((Curry_Prelude.=?=) e y d (addCs c cs))
  (=?=) (Fail_C_CVisibility cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CVisibility cd i x y) d cs = narrow cd i
    ((Curry_Prelude.=?=) z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CVisibility cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CVisibility cd c e) d cs = guardCons cd c
    ((Curry_Prelude.=?=) y e d (addCs c cs))
  (=?=) _ (Fail_C_CVisibility cd info) _ _ = failCons cd info
  (=?=) C_Public C_Public d cs = Curry_Prelude.C_True
  (=?=) C_Private C_Private d cs = Curry_Prelude.C_True
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CVisibility cd i x y) z d cs = narrow cd i
    ((Curry_Prelude.<?=) x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CVisibility cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CVisibility cd c e) y d cs = guardCons cd c
    ((Curry_Prelude.<?=) e y d (addCs c cs))
  (<?=) (Fail_C_CVisibility cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CVisibility cd i x y) d cs = narrow cd i
    ((Curry_Prelude.<?=) z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CVisibility cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CVisibility cd c e) d cs = guardCons cd c
    ((Curry_Prelude.<?=) y e d (addCs c cs))
  (<?=) _ (Fail_C_CVisibility cd info) _ _ = failCons cd info
  (<?=) C_Public C_Public d cs = Curry_Prelude.C_True
  (<?=) C_Public C_Private _ _ = Curry_Prelude.C_True
  (<?=) C_Private C_Private d cs = Curry_Prelude.C_True
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CurryProg
  = C_CurryProg (Curry_Prelude.OP_List Curry_Prelude.C_Char)
    (Curry_Prelude.OP_List (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    (Curry_Prelude.OP_List C_CTypeDecl) (Curry_Prelude.OP_List C_CFuncDecl)
    (Curry_Prelude.OP_List C_COpDecl)
  | Choice_C_CurryProg Cover ID C_CurryProg C_CurryProg
  | Choices_C_CurryProg Cover ID [C_CurryProg]
  | Fail_C_CurryProg Cover FailInfo
  | Guard_C_CurryProg Cover Constraints C_CurryProg

instance Show C_CurryProg where
  showsPrec d (Choice_C_CurryProg cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CurryProg cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CurryProg cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CurryProg cd info) = showChar '!'
  showsPrec _ (C_CurryProg x1 x2 x3 x4 x5) = showString "(CurryProg" . (showChar
    ' ' . (shows x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 .
    (showChar ' ' . (shows x4 . (showChar ' ' . (shows x5 . showChar
    ')'))))))))))

instance Read C_CurryProg where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CurryProg x1 x2 x3 x4 x5
    , r5) | (_, r0) <- readQualified "AbstractCurry" "CurryProg" r, (x1, r1) <-
    readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2,
    (x4, r4) <- readsPrec 11 r3, (x5, r5) <- readsPrec 11 r4]) s

instance NonDet C_CurryProg where
  choiceCons = Choice_C_CurryProg
  choicesCons = Choices_C_CurryProg
  failCons = Fail_C_CurryProg
  guardCons = Guard_C_CurryProg
  try (Choice_C_CurryProg cd i x y) = tryChoice cd i x y
  try (Choices_C_CurryProg cd i xs) = tryChoices cd i xs
  try (Fail_C_CurryProg cd info) = Fail cd info
  try (Guard_C_CurryProg cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CurryProg cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CurryProg cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CurryProg cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CurryProg cd i _) = error
    ("AbstractCurry.CurryProg.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CurryProg cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CurryProg cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CurryProg where
  generate s c = Choices_C_CurryProg c (freeID [5] s) [C_CurryProg (generate
    (leftSupply (leftSupply (leftSupply s))) c) (generate (rightSupply
    (leftSupply (leftSupply s))) c) (generate (rightSupply (leftSupply s)) c)
    (generate (leftSupply (rightSupply s)) c) (generate (rightSupply
    (rightSupply s)) c)]

instance NormalForm C_CurryProg where
  ($!!) cont (C_CurryProg x1 x2 x3 x4 x5) d cs = ($!!) (\y1 d cs -> ($!!)
    (\y2 d cs -> ($!!) (\y3 d cs -> ($!!) (\y4 d cs -> ($!!) (\y5 d cs -> cont
    (C_CurryProg y1 y2 y3 y4 y5) d cs) x5 d cs) x4 d cs) x3 d cs) x2 d cs) x1 d
    cs
  ($!!) cont (Choice_C_CurryProg cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CurryProg cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CurryProg cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CurryProg cd info) _ _ = failCons cd info
  ($##) cont (C_CurryProg x1 x2 x3 x4 x5) d cs = ($##) (\y1 d cs -> ($##)
    (\y2 d cs -> ($##) (\y3 d cs -> ($##) (\y4 d cs -> ($##) (\y5 d cs -> cont
    (C_CurryProg y1 y2 y3 y4 y5) d cs) x5 d cs) x4 d cs) x3 d cs) x2 d cs) x1 d
    cs
  ($##) cont (Choice_C_CurryProg cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CurryProg cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CurryProg cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CurryProg cd info) _ _ = failCons cd info
  showCons (C_CurryProg _ _ _ _ _) = "AbstractCurry.CurryProg _ _ _ _ _"
  showCons x = error ("AbstractCurry.CurryProg.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CurryProg x1 x2 x3 x4 x5) = search (\y1 -> search
    (\y2 -> search (\y3 -> search (\y4 -> search (\y5 -> cont (C_CurryProg y1 y2
    y3 y4 y5)) x5) x4) x3) x2) x1
  searchNF _ _ x = error
    ("AbstractCurry.CurryProg.searchNF: no constructor: " ++ show x)

instance Unifiable C_CurryProg where
  (=.=) (C_CurryProg x1 x2 x3 x4 x5) (C_CurryProg y1 y2 y3 y4 y5) d cs = (&)
    ((=:=) x1 y1 d cs) ((&) ((=:=) x2 y2 d cs) ((&) ((=:=) x3 y3 d cs) ((&)
    ((=:=) x4 y4 d cs) ((=:=) x5 y5 d cs) d cs) d cs) d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CurryProg x1 x2 x3 x4 x5) (C_CurryProg y1 y2 y3 y4 y5) d cs = (&)
    ((=:<=) x1 y1 d cs) ((&) ((=:<=) x2 y2 d cs) ((&) ((=:<=) x3 y3 d cs) ((&)
    ((=:<=) x4 y4 d cs) ((=:<=) x5 y5 d cs) d cs) d cs) d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CurryProg x3 x4 x5 x6 x7) = (i :=: ChooseN 0 5) : concat [bind cd
    (leftID (leftID (leftID i))) x3, bind cd (rightID (leftID (leftID i))) x4
    , bind cd (rightID (leftID i)) x5, bind cd (leftID (rightID i)) x6, bind cd
    (rightID (rightID i)) x7]
  bind d i (Choice_C_CurryProg cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CurryProg cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CurryProg cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CurryProg cd i _) = error
    ("AbstractCurry.CurryProg.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CurryProg cd info) = [Unsolvable info]
  bind d i (Guard_C_CurryProg cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CurryProg x3 x4 x5 x6 x7) = (i :=: ChooseN 0 5) : [leftID
    (leftID (leftID i)) :=: LazyBind (lazyBind cd (leftID (leftID (leftID i)))
    x3), rightID (leftID (leftID i)) :=: LazyBind (lazyBind cd (rightID (leftID
    (leftID i))) x4), rightID (leftID i) :=: LazyBind (lazyBind cd (rightID
    (leftID i)) x5), leftID (rightID i) :=: LazyBind (lazyBind cd (leftID
    (rightID i)) x6), rightID (rightID i) :=: LazyBind (lazyBind cd (rightID
    (rightID i)) x7)]
  lazyBind d i (Choice_C_CurryProg cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CurryProg cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CurryProg cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CurryProg cd i _) = error
    ("AbstractCurry.CurryProg.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CurryProg cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CurryProg cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CurryProg where
  (=?=) (Choice_C_CurryProg cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CurryProg cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CurryProg cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CurryProg cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CurryProg cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CurryProg cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CurryProg cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CurryProg cd info) _ _ = failCons cd info
  (=?=) (C_CurryProg x1 x2 x3 x4 x5) (C_CurryProg y1 y2 y3 y4 y5) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x4 y4 d cs)
    ((Curry_Prelude.=?=) x5 y5 d cs) d cs) d cs) d cs) d cs
  (<?=) (Choice_C_CurryProg cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CurryProg cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CurryProg cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CurryProg cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CurryProg cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CurryProg cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CurryProg cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CurryProg cd info) _ _ = failCons cd info
  (<?=) (C_CurryProg x1 x2 x3 x4 x5) (C_CurryProg y1 y2 y3 y4 y5) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x4 y4 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x4 y4 d cs)
    ((Curry_Prelude.<?=) x5 y5 d cs) d cs) d cs) d cs) d cs) d cs) d cs) d cs) d
    cs

data C_CTypeDecl
  = C_CType (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    C_CVisibility (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2
    Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char)))
    (Curry_Prelude.OP_List C_CConsDecl)
  | C_CTypeSyn (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    C_CVisibility (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2
    Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char)))
    C_CTypeExpr
  | C_CNewType (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    C_CVisibility (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2
    Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char)))
    C_CConsDecl
  | Choice_C_CTypeDecl Cover ID C_CTypeDecl C_CTypeDecl
  | Choices_C_CTypeDecl Cover ID [C_CTypeDecl]
  | Fail_C_CTypeDecl Cover FailInfo
  | Guard_C_CTypeDecl Cover Constraints C_CTypeDecl

instance Show C_CTypeDecl where
  showsPrec d (Choice_C_CTypeDecl cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CTypeDecl cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CTypeDecl cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CTypeDecl cd info) = showChar '!'
  showsPrec _ (C_CType x1 x2 x3 x4) = showString "(CType" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 .
    (showChar ' ' . (shows x4 . showChar ')'))))))))
  showsPrec _ (C_CTypeSyn x1 x2 x3 x4) = showString "(CTypeSyn" . (showChar
    ' ' . (shows x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 .
    (showChar ' ' . (shows x4 . showChar ')'))))))))
  showsPrec _ (C_CNewType x1 x2 x3 x4) = showString "(CNewType" . (showChar
    ' ' . (shows x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 .
    (showChar ' ' . (shows x4 . showChar ')'))))))))

instance Read C_CTypeDecl where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CType x1 x2 x3 x4, r4) | (_
    , r0) <- readQualified "AbstractCurry" "CType" r, (x1, r1) <- readsPrec 11
    r0, (x2, r2) <- readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2, (x4, r4) <-
    readsPrec 11 r3]) s ++ (readParen ((>) d 10) (\r -> [(C_CTypeSyn x1 x2 x3 x4
    , r4) | (_, r0) <- readQualified "AbstractCurry" "CTypeSyn" r, (x1, r1) <-
    readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2,
    (x4, r4) <- readsPrec 11 r3]) s ++ readParen ((>) d 10) (\r -> [(C_CNewType
    x1 x2 x3 x4, r4) | (_, r0) <- readQualified "AbstractCurry" "CNewType" r,
    (x1, r1) <- readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1, (x3, r3) <-
    readsPrec 11 r2, (x4, r4) <- readsPrec 11 r3]) s)

instance NonDet C_CTypeDecl where
  choiceCons = Choice_C_CTypeDecl
  choicesCons = Choices_C_CTypeDecl
  failCons = Fail_C_CTypeDecl
  guardCons = Guard_C_CTypeDecl
  try (Choice_C_CTypeDecl cd i x y) = tryChoice cd i x y
  try (Choices_C_CTypeDecl cd i xs) = tryChoices cd i xs
  try (Fail_C_CTypeDecl cd info) = Fail cd info
  try (Guard_C_CTypeDecl cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CTypeDecl cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CTypeDecl cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CTypeDecl cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CTypeDecl cd i _) = error
    ("AbstractCurry.CTypeDecl.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CTypeDecl cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CTypeDecl cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CTypeDecl where
  generate s c = Choices_C_CTypeDecl c (freeID [4, 4, 4] s) [C_CType (generate
    (leftSupply (leftSupply s)) c) (generate (rightSupply (leftSupply s)) c)
    (generate (leftSupply (rightSupply s)) c) (generate (rightSupply
    (rightSupply s)) c), C_CTypeSyn (generate (leftSupply (leftSupply s)) c)
    (generate (rightSupply (leftSupply s)) c) (generate (leftSupply (rightSupply
    s)) c) (generate (rightSupply (rightSupply s)) c), C_CNewType (generate
    (leftSupply (leftSupply s)) c) (generate (rightSupply (leftSupply s)) c)
    (generate (leftSupply (rightSupply s)) c) (generate (rightSupply
    (rightSupply s)) c)]

instance NormalForm C_CTypeDecl where
  ($!!) cont (C_CType x1 x2 x3 x4) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    ($!!) (\y3 d cs -> ($!!) (\y4 d cs -> cont (C_CType y1 y2 y3 y4) d cs) x4 d
    cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CTypeSyn x1 x2 x3 x4) d cs = ($!!) (\y1 d cs -> ($!!)
    (\y2 d cs -> ($!!) (\y3 d cs -> ($!!) (\y4 d cs -> cont (C_CTypeSyn y1 y2 y3
    y4) d cs) x4 d cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CNewType x1 x2 x3 x4) d cs = ($!!) (\y1 d cs -> ($!!)
    (\y2 d cs -> ($!!) (\y3 d cs -> ($!!) (\y4 d cs -> cont (C_CNewType y1 y2 y3
    y4) d cs) x4 d cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CTypeDecl cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CTypeDecl cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CTypeDecl cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CTypeDecl cd info) _ _ = failCons cd info
  ($##) cont (C_CType x1 x2 x3 x4) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    ($##) (\y3 d cs -> ($##) (\y4 d cs -> cont (C_CType y1 y2 y3 y4) d cs) x4 d
    cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (C_CTypeSyn x1 x2 x3 x4) d cs = ($##) (\y1 d cs -> ($##)
    (\y2 d cs -> ($##) (\y3 d cs -> ($##) (\y4 d cs -> cont (C_CTypeSyn y1 y2 y3
    y4) d cs) x4 d cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (C_CNewType x1 x2 x3 x4) d cs = ($##) (\y1 d cs -> ($##)
    (\y2 d cs -> ($##) (\y3 d cs -> ($##) (\y4 d cs -> cont (C_CNewType y1 y2 y3
    y4) d cs) x4 d cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CTypeDecl cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CTypeDecl cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CTypeDecl cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CTypeDecl cd info) _ _ = failCons cd info
  showCons (C_CType _ _ _ _) = "AbstractCurry.CType _ _ _ _"
  showCons (C_CTypeSyn _ _ _ _) = "AbstractCurry.CTypeSyn _ _ _ _"
  showCons (C_CNewType _ _ _ _) = "AbstractCurry.CNewType _ _ _ _"
  showCons x = error ("AbstractCurry.CTypeDecl.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CType x1 x2 x3 x4) = search (\y1 -> search (\y2 ->
    search (\y3 -> search (\y4 -> cont (C_CType y1 y2 y3 y4)) x4) x3) x2) x1
  searchNF search cont (C_CTypeSyn x1 x2 x3 x4) = search (\y1 -> search (\y2 ->
    search (\y3 -> search (\y4 -> cont (C_CTypeSyn y1 y2 y3 y4)) x4) x3) x2) x1
  searchNF search cont (C_CNewType x1 x2 x3 x4) = search (\y1 -> search (\y2 ->
    search (\y3 -> search (\y4 -> cont (C_CNewType y1 y2 y3 y4)) x4) x3) x2) x1
  searchNF _ _ x = error
    ("AbstractCurry.CTypeDecl.searchNF: no constructor: " ++ show x)

instance Unifiable C_CTypeDecl where
  (=.=) (C_CType x1 x2 x3 x4) (C_CType y1 y2 y3 y4) d cs = (&) ((=:=) x1 y1 d
    cs) ((&) ((=:=) x2 y2 d cs) ((&) ((=:=) x3 y3 d cs) ((=:=) x4 y4 d cs) d cs)
    d cs) d cs
  (=.=) (C_CTypeSyn x1 x2 x3 x4) (C_CTypeSyn y1 y2 y3 y4) d cs = (&) ((=:=) x1
    y1 d cs) ((&) ((=:=) x2 y2 d cs) ((&) ((=:=) x3 y3 d cs) ((=:=) x4 y4 d cs)
    d cs) d cs) d cs
  (=.=) (C_CNewType x1 x2 x3 x4) (C_CNewType y1 y2 y3 y4) d cs = (&) ((=:=) x1
    y1 d cs) ((&) ((=:=) x2 y2 d cs) ((&) ((=:=) x3 y3 d cs) ((=:=) x4 y4 d cs)
    d cs) d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CType x1 x2 x3 x4) (C_CType y1 y2 y3 y4) d cs = (&) ((=:<=) x1 y1 d
    cs) ((&) ((=:<=) x2 y2 d cs) ((&) ((=:<=) x3 y3 d cs) ((=:<=) x4 y4 d cs) d
    cs) d cs) d cs
  (=.<=) (C_CTypeSyn x1 x2 x3 x4) (C_CTypeSyn y1 y2 y3 y4) d cs = (&) ((=:<=) x1
    y1 d cs) ((&) ((=:<=) x2 y2 d cs) ((&) ((=:<=) x3 y3 d cs) ((=:<=) x4 y4 d
    cs) d cs) d cs) d cs
  (=.<=) (C_CNewType x1 x2 x3 x4) (C_CNewType y1 y2 y3 y4) d cs = (&) ((=:<=) x1
    y1 d cs) ((&) ((=:<=) x2 y2 d cs) ((&) ((=:<=) x3 y3 d cs) ((=:<=) x4 y4 d
    cs) d cs) d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CType x3 x4 x5 x6) = (i :=: ChooseN 0 4) : concat [bind cd
    (leftID (leftID i)) x3, bind cd (rightID (leftID i)) x4, bind cd (leftID
    (rightID i)) x5, bind cd (rightID (rightID i)) x6]
  bind cd i (C_CTypeSyn x3 x4 x5 x6) = (i :=: ChooseN 1 4) : concat [bind cd
    (leftID (leftID i)) x3, bind cd (rightID (leftID i)) x4, bind cd (leftID
    (rightID i)) x5, bind cd (rightID (rightID i)) x6]
  bind cd i (C_CNewType x3 x4 x5 x6) = (i :=: ChooseN 2 4) : concat [bind cd
    (leftID (leftID i)) x3, bind cd (rightID (leftID i)) x4, bind cd (leftID
    (rightID i)) x5, bind cd (rightID (rightID i)) x6]
  bind d i (Choice_C_CTypeDecl cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CTypeDecl cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CTypeDecl cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CTypeDecl cd i _) = error
    ("AbstractCurry.CTypeDecl.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CTypeDecl cd info) = [Unsolvable info]
  bind d i (Guard_C_CTypeDecl cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CType x3 x4 x5 x6) = (i :=: ChooseN 0 4) : [leftID (leftID
    i) :=: LazyBind (lazyBind cd (leftID (leftID i)) x3), rightID (leftID i) :=:
    LazyBind (lazyBind cd (rightID (leftID i)) x4), leftID (rightID i) :=:
    LazyBind (lazyBind cd (leftID (rightID i)) x5), rightID (rightID i) :=:
    LazyBind (lazyBind cd (rightID (rightID i)) x6)]
  lazyBind cd i (C_CTypeSyn x3 x4 x5 x6) = (i :=: ChooseN 1 4) : [leftID (leftID
    i) :=: LazyBind (lazyBind cd (leftID (leftID i)) x3), rightID (leftID i) :=:
    LazyBind (lazyBind cd (rightID (leftID i)) x4), leftID (rightID i) :=:
    LazyBind (lazyBind cd (leftID (rightID i)) x5), rightID (rightID i) :=:
    LazyBind (lazyBind cd (rightID (rightID i)) x6)]
  lazyBind cd i (C_CNewType x3 x4 x5 x6) = (i :=: ChooseN 2 4) : [leftID (leftID
    i) :=: LazyBind (lazyBind cd (leftID (leftID i)) x3), rightID (leftID i) :=:
    LazyBind (lazyBind cd (rightID (leftID i)) x4), leftID (rightID i) :=:
    LazyBind (lazyBind cd (leftID (rightID i)) x5), rightID (rightID i) :=:
    LazyBind (lazyBind cd (rightID (rightID i)) x6)]
  lazyBind d i (Choice_C_CTypeDecl cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CTypeDecl cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CTypeDecl cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CTypeDecl cd i _) = error
    ("AbstractCurry.CTypeDecl.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CTypeDecl cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CTypeDecl cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CTypeDecl where
  (=?=) (Choice_C_CTypeDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CTypeDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CTypeDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CTypeDecl cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CTypeDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CTypeDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CTypeDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CTypeDecl cd info) _ _ = failCons cd info
  (=?=) (C_CType x1 x2 x3 x4) (C_CType y1 y2 y3 y4) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    ((Curry_Prelude.=?=) x4 y4 d cs) d cs) d cs) d cs
  (=?=) (C_CTypeSyn x1 x2 x3 x4) (C_CTypeSyn y1 y2 y3 y4) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    ((Curry_Prelude.=?=) x4 y4 d cs) d cs) d cs) d cs
  (=?=) (C_CNewType x1 x2 x3 x4) (C_CNewType y1 y2 y3 y4) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    ((Curry_Prelude.=?=) x4 y4 d cs) d cs) d cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CTypeDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CTypeDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CTypeDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CTypeDecl cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CTypeDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CTypeDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CTypeDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CTypeDecl cd info) _ _ = failCons cd info
  (<?=) (C_CType x1 x2 x3 x4) (C_CType y1 y2 y3 y4) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    ((Curry_Prelude.<?=) x4 y4 d cs) d cs) d cs) d cs) d cs) d cs) d cs
  (<?=) (C_CType _ _ _ _) (C_CTypeSyn _ _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CType _ _ _ _) (C_CNewType _ _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CTypeSyn x1 x2 x3 x4) (C_CTypeSyn y1 y2 y3 y4) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    ((Curry_Prelude.<?=) x4 y4 d cs) d cs) d cs) d cs) d cs) d cs) d cs
  (<?=) (C_CTypeSyn _ _ _ _) (C_CNewType _ _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CNewType x1 x2 x3 x4) (C_CNewType y1 y2 y3 y4) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    ((Curry_Prelude.<?=) x4 y4 d cs) d cs) d cs) d cs) d cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CConsDecl
  = C_CCons (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    C_CVisibility (Curry_Prelude.OP_List C_CTypeExpr)
  | C_CRecord (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    C_CVisibility (Curry_Prelude.OP_List C_CFieldDecl)
  | Choice_C_CConsDecl Cover ID C_CConsDecl C_CConsDecl
  | Choices_C_CConsDecl Cover ID [C_CConsDecl]
  | Fail_C_CConsDecl Cover FailInfo
  | Guard_C_CConsDecl Cover Constraints C_CConsDecl

instance Show C_CConsDecl where
  showsPrec d (Choice_C_CConsDecl cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CConsDecl cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CConsDecl cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CConsDecl cd info) = showChar '!'
  showsPrec _ (C_CCons x1 x2 x3) = showString "(CCons" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 . showChar
    ')'))))))
  showsPrec _ (C_CRecord x1 x2 x3) = showString "(CRecord" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 . showChar
    ')'))))))

instance Read C_CConsDecl where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CCons x1 x2 x3, r3) | (_, r0)
    <- readQualified "AbstractCurry" "CCons" r, (x1, r1) <- readsPrec 11 r0, (x2
    , r2) <- readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2]) s ++ readParen ((>)
    d 10) (\r -> [(C_CRecord x1 x2 x3, r3) | (_, r0) <- readQualified
    "AbstractCurry" "CRecord" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <-
    readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2]) s

instance NonDet C_CConsDecl where
  choiceCons = Choice_C_CConsDecl
  choicesCons = Choices_C_CConsDecl
  failCons = Fail_C_CConsDecl
  guardCons = Guard_C_CConsDecl
  try (Choice_C_CConsDecl cd i x y) = tryChoice cd i x y
  try (Choices_C_CConsDecl cd i xs) = tryChoices cd i xs
  try (Fail_C_CConsDecl cd info) = Fail cd info
  try (Guard_C_CConsDecl cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CConsDecl cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CConsDecl cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CConsDecl cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CConsDecl cd i _) = error
    ("AbstractCurry.CConsDecl.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CConsDecl cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CConsDecl cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CConsDecl where
  generate s c = Choices_C_CConsDecl c (freeID [3, 3] s) [C_CCons (generate
    (leftSupply (leftSupply s)) c) (generate (rightSupply (leftSupply s)) c)
    (generate (rightSupply s) c), C_CRecord (generate (leftSupply (leftSupply
    s)) c) (generate (rightSupply (leftSupply s)) c) (generate (rightSupply s)
    c)]

instance NormalForm C_CConsDecl where
  ($!!) cont (C_CCons x1 x2 x3) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    ($!!) (\y3 d cs -> cont (C_CCons y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CRecord x1 x2 x3) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    ($!!) (\y3 d cs -> cont (C_CRecord y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CConsDecl cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CConsDecl cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CConsDecl cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CConsDecl cd info) _ _ = failCons cd info
  ($##) cont (C_CCons x1 x2 x3) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    ($##) (\y3 d cs -> cont (C_CCons y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (C_CRecord x1 x2 x3) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    ($##) (\y3 d cs -> cont (C_CRecord y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CConsDecl cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CConsDecl cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CConsDecl cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CConsDecl cd info) _ _ = failCons cd info
  showCons (C_CCons _ _ _) = "AbstractCurry.CCons _ _ _"
  showCons (C_CRecord _ _ _) = "AbstractCurry.CRecord _ _ _"
  showCons x = error ("AbstractCurry.CConsDecl.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CCons x1 x2 x3) = search (\y1 -> search (\y2 -> search
    (\y3 -> cont (C_CCons y1 y2 y3)) x3) x2) x1
  searchNF search cont (C_CRecord x1 x2 x3) = search (\y1 -> search (\y2 ->
    search (\y3 -> cont (C_CRecord y1 y2 y3)) x3) x2) x1
  searchNF _ _ x = error
    ("AbstractCurry.CConsDecl.searchNF: no constructor: " ++ show x)

instance Unifiable C_CConsDecl where
  (=.=) (C_CCons x1 x2 x3) (C_CCons y1 y2 y3) d cs = (&) ((=:=) x1 y1 d cs) ((&)
    ((=:=) x2 y2 d cs) ((=:=) x3 y3 d cs) d cs) d cs
  (=.=) (C_CRecord x1 x2 x3) (C_CRecord y1 y2 y3) d cs = (&) ((=:=) x1 y1 d cs)
    ((&) ((=:=) x2 y2 d cs) ((=:=) x3 y3 d cs) d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CCons x1 x2 x3) (C_CCons y1 y2 y3) d cs = (&) ((=:<=) x1 y1 d cs)
    ((&) ((=:<=) x2 y2 d cs) ((=:<=) x3 y3 d cs) d cs) d cs
  (=.<=) (C_CRecord x1 x2 x3) (C_CRecord y1 y2 y3) d cs = (&) ((=:<=) x1 y1 d
    cs) ((&) ((=:<=) x2 y2 d cs) ((=:<=) x3 y3 d cs) d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CCons x3 x4 x5) = (i :=: ChooseN 0 3) : concat [bind cd (leftID
    (leftID i)) x3, bind cd (rightID (leftID i)) x4, bind cd (rightID i) x5]
  bind cd i (C_CRecord x3 x4 x5) = (i :=: ChooseN 1 3) : concat [bind cd (leftID
    (leftID i)) x3, bind cd (rightID (leftID i)) x4, bind cd (rightID i) x5]
  bind d i (Choice_C_CConsDecl cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CConsDecl cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CConsDecl cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CConsDecl cd i _) = error
    ("AbstractCurry.CConsDecl.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CConsDecl cd info) = [Unsolvable info]
  bind d i (Guard_C_CConsDecl cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CCons x3 x4 x5) = (i :=: ChooseN 0 3) : [leftID (leftID
    i) :=: LazyBind (lazyBind cd (leftID (leftID i)) x3), rightID (leftID i) :=:
    LazyBind (lazyBind cd (rightID (leftID i)) x4), rightID i :=: LazyBind
    (lazyBind cd (rightID i) x5)]
  lazyBind cd i (C_CRecord x3 x4 x5) = (i :=: ChooseN 1 3) : [leftID (leftID
    i) :=: LazyBind (lazyBind cd (leftID (leftID i)) x3), rightID (leftID i) :=:
    LazyBind (lazyBind cd (rightID (leftID i)) x4), rightID i :=: LazyBind
    (lazyBind cd (rightID i) x5)]
  lazyBind d i (Choice_C_CConsDecl cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CConsDecl cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CConsDecl cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CConsDecl cd i _) = error
    ("AbstractCurry.CConsDecl.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CConsDecl cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CConsDecl cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CConsDecl where
  (=?=) (Choice_C_CConsDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CConsDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CConsDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CConsDecl cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CConsDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CConsDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CConsDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CConsDecl cd info) _ _ = failCons cd info
  (=?=) (C_CCons x1 x2 x3) (C_CCons y1 y2 y3) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x2 y2 d cs) ((Curry_Prelude.=?=) x3 y3 d cs) d cs) d cs
  (=?=) (C_CRecord x1 x2 x3) (C_CRecord y1 y2 y3) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    ((Curry_Prelude.=?=) x3 y3 d cs) d cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CConsDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CConsDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CConsDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CConsDecl cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CConsDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CConsDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CConsDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CConsDecl cd info) _ _ = failCons cd info
  (<?=) (C_CCons x1 x2 x3) (C_CCons y1 y2 y3) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) (Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x2 y2 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x2 y2 d cs) ((Curry_Prelude.<?=) x3 y3 d cs) d cs) d
    cs) d cs) d cs
  (<?=) (C_CCons _ _ _) (C_CRecord _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CRecord x1 x2 x3) (C_CRecord y1 y2 y3) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    ((Curry_Prelude.<?=) x3 y3 d cs) d cs) d cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CFieldDecl
  = C_CField (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    C_CVisibility C_CTypeExpr
  | Choice_C_CFieldDecl Cover ID C_CFieldDecl C_CFieldDecl
  | Choices_C_CFieldDecl Cover ID [C_CFieldDecl]
  | Fail_C_CFieldDecl Cover FailInfo
  | Guard_C_CFieldDecl Cover Constraints C_CFieldDecl

instance Show C_CFieldDecl where
  showsPrec d (Choice_C_CFieldDecl cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CFieldDecl cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CFieldDecl cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CFieldDecl cd info) = showChar '!'
  showsPrec _ (C_CField x1 x2 x3) = showString "(CField" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 . showChar
    ')'))))))

instance Read C_CFieldDecl where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CField x1 x2 x3, r3) | (_, r0)
    <- readQualified "AbstractCurry" "CField" r, (x1, r1) <- readsPrec 11 r0,
    (x2, r2) <- readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2]) s

instance NonDet C_CFieldDecl where
  choiceCons = Choice_C_CFieldDecl
  choicesCons = Choices_C_CFieldDecl
  failCons = Fail_C_CFieldDecl
  guardCons = Guard_C_CFieldDecl
  try (Choice_C_CFieldDecl cd i x y) = tryChoice cd i x y
  try (Choices_C_CFieldDecl cd i xs) = tryChoices cd i xs
  try (Fail_C_CFieldDecl cd info) = Fail cd info
  try (Guard_C_CFieldDecl cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CFieldDecl cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CFieldDecl cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CFieldDecl cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CFieldDecl cd i _) = error
    ("AbstractCurry.CFieldDecl.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CFieldDecl cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CFieldDecl cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CFieldDecl where
  generate s c = Choices_C_CFieldDecl c (freeID [3] s) [C_CField (generate
    (leftSupply (leftSupply s)) c) (generate (rightSupply (leftSupply s)) c)
    (generate (rightSupply s) c)]

instance NormalForm C_CFieldDecl where
  ($!!) cont (C_CField x1 x2 x3) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    ($!!) (\y3 d cs -> cont (C_CField y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CFieldDecl cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CFieldDecl cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CFieldDecl cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CFieldDecl cd info) _ _ = failCons cd info
  ($##) cont (C_CField x1 x2 x3) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    ($##) (\y3 d cs -> cont (C_CField y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CFieldDecl cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CFieldDecl cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CFieldDecl cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CFieldDecl cd info) _ _ = failCons cd info
  showCons (C_CField _ _ _) = "AbstractCurry.CField _ _ _"
  showCons x = error ("AbstractCurry.CFieldDecl.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CField x1 x2 x3) = search (\y1 -> search (\y2 ->
    search (\y3 -> cont (C_CField y1 y2 y3)) x3) x2) x1
  searchNF _ _ x = error
    ("AbstractCurry.CFieldDecl.searchNF: no constructor: " ++ show x)

instance Unifiable C_CFieldDecl where
  (=.=) (C_CField x1 x2 x3) (C_CField y1 y2 y3) d cs = (&) ((=:=) x1 y1 d cs)
    ((&) ((=:=) x2 y2 d cs) ((=:=) x3 y3 d cs) d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CField x1 x2 x3) (C_CField y1 y2 y3) d cs = (&) ((=:<=) x1 y1 d cs)
    ((&) ((=:<=) x2 y2 d cs) ((=:<=) x3 y3 d cs) d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CField x3 x4 x5) = (i :=: ChooseN 0 3) : concat [bind cd (leftID
    (leftID i)) x3, bind cd (rightID (leftID i)) x4, bind cd (rightID i) x5]
  bind d i (Choice_C_CFieldDecl cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CFieldDecl cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j
    xs
  bind d i (Choices_C_CFieldDecl cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CFieldDecl cd i _) = error
    ("AbstractCurry.CFieldDecl.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CFieldDecl cd info) = [Unsolvable info]
  bind d i (Guard_C_CFieldDecl cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CField x3 x4 x5) = (i :=: ChooseN 0 3) : [leftID (leftID
    i) :=: LazyBind (lazyBind cd (leftID (leftID i)) x3), rightID (leftID i) :=:
    LazyBind (lazyBind cd (rightID (leftID i)) x4), rightID i :=: LazyBind
    (lazyBind cd (rightID i) x5)]
  lazyBind d i (Choice_C_CFieldDecl cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CFieldDecl cd j@(FreeID _ _) xs) = lazyBindOrNarrow d
    i cd j xs
  lazyBind d i (Choices_C_CFieldDecl cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CFieldDecl cd i _) = error
    ("AbstractCurry.CFieldDecl.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CFieldDecl cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CFieldDecl cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CFieldDecl where
  (=?=) (Choice_C_CFieldDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CFieldDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CFieldDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CFieldDecl cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CFieldDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CFieldDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CFieldDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CFieldDecl cd info) _ _ = failCons cd info
  (=?=) (C_CField x1 x2 x3) (C_CField y1 y2 y3) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    ((Curry_Prelude.=?=) x3 y3 d cs) d cs) d cs
  (<?=) (Choice_C_CFieldDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CFieldDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CFieldDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CFieldDecl cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CFieldDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CFieldDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CFieldDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CFieldDecl cd info) _ _ = failCons cd info
  (<?=) (C_CField x1 x2 x3) (C_CField y1 y2 y3) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    ((Curry_Prelude.<?=) x3 y3 d cs) d cs) d cs) d cs) d cs

data C_CTypeExpr
  = C_CTVar (Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List
    Curry_Prelude.C_Char))
  | C_CFuncType C_CTypeExpr C_CTypeExpr
  | C_CTCons (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    (Curry_Prelude.OP_List C_CTypeExpr)
  | Choice_C_CTypeExpr Cover ID C_CTypeExpr C_CTypeExpr
  | Choices_C_CTypeExpr Cover ID [C_CTypeExpr]
  | Fail_C_CTypeExpr Cover FailInfo
  | Guard_C_CTypeExpr Cover Constraints C_CTypeExpr

instance Show C_CTypeExpr where
  showsPrec d (Choice_C_CTypeExpr cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CTypeExpr cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CTypeExpr cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CTypeExpr cd info) = showChar '!'
  showsPrec _ (C_CTVar x1) = showString "(CTVar" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CFuncType x1 x2) = showString "(CFuncType" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CTCons x1 x2) = showString "(CTCons" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . showChar ')'))))

instance Read C_CTypeExpr where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CTVar x1, r1) | (_, r0) <-
    readQualified "AbstractCurry" "CTVar" r, (x1, r1) <- readsPrec 11 r0]) s ++
    (readParen ((>) d 10) (\r -> [(C_CFuncType x1 x2, r2) | (_, r0) <-
    readQualified "AbstractCurry" "CFuncType" r, (x1, r1) <- readsPrec 11 r0,
    (x2, r2) <- readsPrec 11 r1]) s ++ readParen ((>) d 10) (\r -> [(C_CTCons x1
    x2, r2) | (_, r0) <- readQualified "AbstractCurry" "CTCons" r, (x1, r1) <-
    readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1]) s)

instance NonDet C_CTypeExpr where
  choiceCons = Choice_C_CTypeExpr
  choicesCons = Choices_C_CTypeExpr
  failCons = Fail_C_CTypeExpr
  guardCons = Guard_C_CTypeExpr
  try (Choice_C_CTypeExpr cd i x y) = tryChoice cd i x y
  try (Choices_C_CTypeExpr cd i xs) = tryChoices cd i xs
  try (Fail_C_CTypeExpr cd info) = Fail cd info
  try (Guard_C_CTypeExpr cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CTypeExpr cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CTypeExpr cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CTypeExpr cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CTypeExpr cd i _) = error
    ("AbstractCurry.CTypeExpr.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CTypeExpr cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CTypeExpr cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CTypeExpr where
  generate s c = Choices_C_CTypeExpr c (freeID [1, 2, 2] s) [C_CTVar (generate
    (leftSupply s) c), C_CFuncType (generate (leftSupply s) c) (generate
    (rightSupply s) c), C_CTCons (generate (leftSupply s) c) (generate
    (rightSupply s) c)]

instance NormalForm C_CTypeExpr where
  ($!!) cont (C_CTVar x1) d cs = ($!!) (\y1 d cs -> cont (C_CTVar y1) d cs) x1 d
    cs
  ($!!) cont (C_CFuncType x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CFuncType y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CTCons x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> cont
    (C_CTCons y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CTypeExpr cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CTypeExpr cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CTypeExpr cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CTypeExpr cd info) _ _ = failCons cd info
  ($##) cont (C_CTVar x1) d cs = ($##) (\y1 d cs -> cont (C_CTVar y1) d cs) x1 d
    cs
  ($##) cont (C_CFuncType x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CFuncType y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CTCons x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> cont
    (C_CTCons y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CTypeExpr cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CTypeExpr cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CTypeExpr cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CTypeExpr cd info) _ _ = failCons cd info
  showCons (C_CTVar _) = "AbstractCurry.CTVar _"
  showCons (C_CFuncType _ _) = "AbstractCurry.CFuncType _ _"
  showCons (C_CTCons _ _) = "AbstractCurry.CTCons _ _"
  showCons x = error ("AbstractCurry.CTypeExpr.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CTVar x1) = search (\y1 -> cont (C_CTVar y1)) x1
  searchNF search cont (C_CFuncType x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CFuncType y1 y2)) x2) x1
  searchNF search cont (C_CTCons x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CTCons y1 y2)) x2) x1
  searchNF _ _ x = error
    ("AbstractCurry.CTypeExpr.searchNF: no constructor: " ++ show x)

instance Unifiable C_CTypeExpr where
  (=.=) (C_CTVar x1) (C_CTVar y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CFuncType x1 x2) (C_CFuncType y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) (C_CTCons x1 x2) (C_CTCons y1 y2) d cs = (&) ((=:=) x1 y1 d cs) ((=:=)
    x2 y2 d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CTVar x1) (C_CTVar y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CFuncType x1 x2) (C_CFuncType y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CTCons x1 x2) (C_CTCons y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CTVar x3) = (i :=: ChooseN 0 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CFuncType x3 x4) = (i :=: ChooseN 1 2) : concat [bind cd (leftID
    i) x3, bind cd (rightID i) x4]
  bind cd i (C_CTCons x3 x4) = (i :=: ChooseN 2 2) : concat [bind cd (leftID i)
    x3, bind cd (rightID i) x4]
  bind d i (Choice_C_CTypeExpr cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CTypeExpr cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CTypeExpr cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CTypeExpr cd i _) = error
    ("AbstractCurry.CTypeExpr.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CTypeExpr cd info) = [Unsolvable info]
  bind d i (Guard_C_CTypeExpr cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CTVar x3) = (i :=: ChooseN 0 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CFuncType x3 x4) = (i :=: ChooseN 1 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind cd i (C_CTCons x3 x4) = (i :=: ChooseN 2 2) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd (rightID i)
    x4)]
  lazyBind d i (Choice_C_CTypeExpr cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CTypeExpr cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CTypeExpr cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CTypeExpr cd i _) = error
    ("AbstractCurry.CTypeExpr.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CTypeExpr cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CTypeExpr cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CTypeExpr where
  (=?=) (Choice_C_CTypeExpr cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CTypeExpr cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CTypeExpr cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CTypeExpr cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CTypeExpr cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CTypeExpr cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CTypeExpr cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CTypeExpr cd info) _ _ = failCons cd info
  (=?=) (C_CTVar x1) (C_CTVar y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CFuncType x1 x2) (C_CFuncType y1 y2) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CTCons x1 x2) (C_CTCons y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CTypeExpr cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CTypeExpr cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CTypeExpr cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CTypeExpr cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CTypeExpr cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CTypeExpr cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CTypeExpr cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CTypeExpr cd info) _ _ = failCons cd info
  (<?=) (C_CTVar x1) (C_CTVar y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CTVar _) (C_CFuncType _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CTVar _) (C_CTCons _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CFuncType x1 x2) (C_CFuncType y1 y2) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CFuncType _ _) (C_CTCons _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CTCons x1 x2) (C_CTCons y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_COpDecl
  = C_COp (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char)
    (Curry_Prelude.OP_List Curry_Prelude.C_Char)) C_CFixity Curry_Prelude.C_Int
  | Choice_C_COpDecl Cover ID C_COpDecl C_COpDecl
  | Choices_C_COpDecl Cover ID [C_COpDecl]
  | Fail_C_COpDecl Cover FailInfo
  | Guard_C_COpDecl Cover Constraints C_COpDecl

instance Show C_COpDecl where
  showsPrec d (Choice_C_COpDecl cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_COpDecl cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_COpDecl cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_COpDecl cd info) = showChar '!'
  showsPrec _ (C_COp x1 x2 x3) = showString "(COp" . (showChar ' ' . (shows x1 .
    (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 . showChar ')'))))))

instance Read C_COpDecl where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_COp x1 x2 x3, r3) | (_, r0) <-
    readQualified "AbstractCurry" "COp" r, (x1, r1) <- readsPrec 11 r0, (x2, r2)
    <- readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2]) s

instance NonDet C_COpDecl where
  choiceCons = Choice_C_COpDecl
  choicesCons = Choices_C_COpDecl
  failCons = Fail_C_COpDecl
  guardCons = Guard_C_COpDecl
  try (Choice_C_COpDecl cd i x y) = tryChoice cd i x y
  try (Choices_C_COpDecl cd i xs) = tryChoices cd i xs
  try (Fail_C_COpDecl cd info) = Fail cd info
  try (Guard_C_COpDecl cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_COpDecl cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_COpDecl cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_COpDecl cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_COpDecl cd i _) = error
    ("AbstractCurry.COpDecl.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_COpDecl cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_COpDecl cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_COpDecl where
  generate s c = Choices_C_COpDecl c (freeID [3] s) [C_COp (generate (leftSupply
    (leftSupply s)) c) (generate (rightSupply (leftSupply s)) c) (generate
    (rightSupply s) c)]

instance NormalForm C_COpDecl where
  ($!!) cont (C_COp x1 x2 x3) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> ($!!)
    (\y3 d cs -> cont (C_COp y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_COpDecl cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_COpDecl cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_COpDecl cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_COpDecl cd info) _ _ = failCons cd info
  ($##) cont (C_COp x1 x2 x3) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> ($##)
    (\y3 d cs -> cont (C_COp y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_COpDecl cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_COpDecl cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_COpDecl cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_COpDecl cd info) _ _ = failCons cd info
  showCons (C_COp _ _ _) = "AbstractCurry.COp _ _ _"
  showCons x = error ("AbstractCurry.COpDecl.showCons: no constructor: " ++ show
    x)
  searchNF search cont (C_COp x1 x2 x3) = search (\y1 -> search (\y2 -> search
    (\y3 -> cont (C_COp y1 y2 y3)) x3) x2) x1
  searchNF _ _ x = error ("AbstractCurry.COpDecl.searchNF: no constructor: " ++
    show x)

instance Unifiable C_COpDecl where
  (=.=) (C_COp x1 x2 x3) (C_COp y1 y2 y3) d cs = (&) ((=:=) x1 y1 d cs) ((&)
    ((=:=) x2 y2 d cs) ((=:=) x3 y3 d cs) d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_COp x1 x2 x3) (C_COp y1 y2 y3) d cs = (&) ((=:<=) x1 y1 d cs) ((&)
    ((=:<=) x2 y2 d cs) ((=:<=) x3 y3 d cs) d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_COp x3 x4 x5) = (i :=: ChooseN 0 3) : concat [bind cd (leftID
    (leftID i)) x3, bind cd (rightID (leftID i)) x4, bind cd (rightID i) x5]
  bind d i (Choice_C_COpDecl cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_COpDecl cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_COpDecl cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd
    j (map (bind d i) xs)]
  bind _ _ (Choices_C_COpDecl cd i _) = error
    ("AbstractCurry.COpDecl.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_COpDecl cd info) = [Unsolvable info]
  bind d i (Guard_C_COpDecl cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_COp x3 x4 x5) = (i :=: ChooseN 0 3) : [leftID (leftID i) :=:
    LazyBind (lazyBind cd (leftID (leftID i)) x3), rightID (leftID i) :=:
    LazyBind (lazyBind cd (rightID (leftID i)) x4), rightID i :=: LazyBind
    (lazyBind cd (rightID i) x5)]
  lazyBind d i (Choice_C_COpDecl cd j x y) = [ConstraintChoice cd j (lazyBind d
    i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_COpDecl cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_COpDecl cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_COpDecl cd i _) = error
    ("AbstractCurry.COpDecl.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_COpDecl cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_COpDecl cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_COpDecl where
  (=?=) (Choice_C_COpDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=) x
    z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_COpDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_COpDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=) e
    y d (addCs c cs))
  (=?=) (Fail_C_COpDecl cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_COpDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=) z
    x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_COpDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_COpDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=) y
    e d (addCs c cs))
  (=?=) _ (Fail_C_COpDecl cd info) _ _ = failCons cd info
  (=?=) (C_COp x1 x2 x3) (C_COp y1 y2 y3) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x2 y2 d cs) ((Curry_Prelude.=?=) x3 y3 d cs) d cs) d cs
  (<?=) (Choice_C_COpDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=) x
    z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_COpDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_COpDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=) e
    y d (addCs c cs))
  (<?=) (Fail_C_COpDecl cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_COpDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=) z
    x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_COpDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_COpDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=) y
    e d (addCs c cs))
  (<?=) _ (Fail_C_COpDecl cd info) _ _ = failCons cd info
  (<?=) (C_COp x1 x2 x3) (C_COp y1 y2 y3) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) (Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x2 y2 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x2 y2 d cs) ((Curry_Prelude.<?=) x3 y3 d cs) d cs) d
    cs) d cs) d cs

data C_CFixity
  = C_CInfixOp
  | C_CInfixlOp
  | C_CInfixrOp
  | Choice_C_CFixity Cover ID C_CFixity C_CFixity
  | Choices_C_CFixity Cover ID [C_CFixity]
  | Fail_C_CFixity Cover FailInfo
  | Guard_C_CFixity Cover Constraints C_CFixity

instance Show C_CFixity where
  showsPrec d (Choice_C_CFixity cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CFixity cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CFixity cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CFixity cd info) = showChar '!'
  showsPrec _ C_CInfixOp = showString "CInfixOp"
  showsPrec _ C_CInfixlOp = showString "CInfixlOp"
  showsPrec _ C_CInfixrOp = showString "CInfixrOp"

instance Read C_CFixity where
  readsPrec _ s = readParen False (\r -> [(C_CInfixOp, r0) | (_, r0) <-
    readQualified "AbstractCurry" "CInfixOp" r]) s ++ (readParen False (\r ->
    [(C_CInfixlOp, r0) | (_, r0) <- readQualified "AbstractCurry" "CInfixlOp"
    r]) s ++ readParen False (\r -> [(C_CInfixrOp, r0) | (_, r0) <-
    readQualified "AbstractCurry" "CInfixrOp" r]) s)

instance NonDet C_CFixity where
  choiceCons = Choice_C_CFixity
  choicesCons = Choices_C_CFixity
  failCons = Fail_C_CFixity
  guardCons = Guard_C_CFixity
  try (Choice_C_CFixity cd i x y) = tryChoice cd i x y
  try (Choices_C_CFixity cd i xs) = tryChoices cd i xs
  try (Fail_C_CFixity cd info) = Fail cd info
  try (Guard_C_CFixity cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CFixity cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CFixity cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CFixity cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CFixity cd i _) = error
    ("AbstractCurry.CFixity.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CFixity cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CFixity cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CFixity where
  generate s c = Choices_C_CFixity c (freeID [0, 0, 0] s) [C_CInfixOp
    , C_CInfixlOp, C_CInfixrOp]

instance NormalForm C_CFixity where
  ($!!) cont C_CInfixOp d cs = cont C_CInfixOp d cs
  ($!!) cont C_CInfixlOp d cs = cont C_CInfixlOp d cs
  ($!!) cont C_CInfixrOp d cs = cont C_CInfixrOp d cs
  ($!!) cont (Choice_C_CFixity cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CFixity cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CFixity cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CFixity cd info) _ _ = failCons cd info
  ($##) cont C_CInfixOp d cs = cont C_CInfixOp d cs
  ($##) cont C_CInfixlOp d cs = cont C_CInfixlOp d cs
  ($##) cont C_CInfixrOp d cs = cont C_CInfixrOp d cs
  ($##) cont (Choice_C_CFixity cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CFixity cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CFixity cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CFixity cd info) _ _ = failCons cd info
  showCons C_CInfixOp = "AbstractCurry.CInfixOp"
  showCons C_CInfixlOp = "AbstractCurry.CInfixlOp"
  showCons C_CInfixrOp = "AbstractCurry.CInfixrOp"
  showCons x = error ("AbstractCurry.CFixity.showCons: no constructor: " ++ show
    x)
  searchNF _ cont C_CInfixOp = cont C_CInfixOp
  searchNF _ cont C_CInfixlOp = cont C_CInfixlOp
  searchNF _ cont C_CInfixrOp = cont C_CInfixrOp
  searchNF _ _ x = error ("AbstractCurry.CFixity.searchNF: no constructor: " ++
    show x)

instance Unifiable C_CFixity where
  (=.=) C_CInfixOp C_CInfixOp d cs = C_Success
  (=.=) C_CInfixlOp C_CInfixlOp d cs = C_Success
  (=.=) C_CInfixrOp C_CInfixrOp d cs = C_Success
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) C_CInfixOp C_CInfixOp d cs = C_Success
  (=.<=) C_CInfixlOp C_CInfixlOp d cs = C_Success
  (=.<=) C_CInfixrOp C_CInfixrOp d cs = C_Success
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i C_CInfixOp = (i :=: ChooseN 0 0) : concat []
  bind cd i C_CInfixlOp = (i :=: ChooseN 1 0) : concat []
  bind cd i C_CInfixrOp = (i :=: ChooseN 2 0) : concat []
  bind d i (Choice_C_CFixity cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CFixity cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CFixity cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd
    j (map (bind d i) xs)]
  bind _ _ (Choices_C_CFixity cd i _) = error
    ("AbstractCurry.CFixity.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CFixity cd info) = [Unsolvable info]
  bind d i (Guard_C_CFixity cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i C_CInfixOp = (i :=: ChooseN 0 0) : []
  lazyBind cd i C_CInfixlOp = (i :=: ChooseN 1 0) : []
  lazyBind cd i C_CInfixrOp = (i :=: ChooseN 2 0) : []
  lazyBind d i (Choice_C_CFixity cd j x y) = [ConstraintChoice cd j (lazyBind d
    i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CFixity cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CFixity cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CFixity cd i _) = error
    ("AbstractCurry.CFixity.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CFixity cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CFixity cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CFixity where
  (=?=) (Choice_C_CFixity cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=) x
    z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CFixity cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CFixity cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=) e
    y d (addCs c cs))
  (=?=) (Fail_C_CFixity cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CFixity cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=) z
    x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CFixity cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CFixity cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=) y
    e d (addCs c cs))
  (=?=) _ (Fail_C_CFixity cd info) _ _ = failCons cd info
  (=?=) C_CInfixOp C_CInfixOp d cs = Curry_Prelude.C_True
  (=?=) C_CInfixlOp C_CInfixlOp d cs = Curry_Prelude.C_True
  (=?=) C_CInfixrOp C_CInfixrOp d cs = Curry_Prelude.C_True
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CFixity cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=) x
    z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CFixity cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CFixity cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=) e
    y d (addCs c cs))
  (<?=) (Fail_C_CFixity cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CFixity cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=) z
    x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CFixity cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CFixity cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=) y
    e d (addCs c cs))
  (<?=) _ (Fail_C_CFixity cd info) _ _ = failCons cd info
  (<?=) C_CInfixOp C_CInfixOp d cs = Curry_Prelude.C_True
  (<?=) C_CInfixOp C_CInfixlOp _ _ = Curry_Prelude.C_True
  (<?=) C_CInfixOp C_CInfixrOp _ _ = Curry_Prelude.C_True
  (<?=) C_CInfixlOp C_CInfixlOp d cs = Curry_Prelude.C_True
  (<?=) C_CInfixlOp C_CInfixrOp _ _ = Curry_Prelude.C_True
  (<?=) C_CInfixrOp C_CInfixrOp d cs = Curry_Prelude.C_True
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CFuncDecl
  = C_CFunc (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    Curry_Prelude.C_Int C_CVisibility C_CTypeExpr (Curry_Prelude.OP_List
    C_CRule)
  | C_CmtFunc (Curry_Prelude.OP_List Curry_Prelude.C_Char)
    (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char)
    (Curry_Prelude.OP_List Curry_Prelude.C_Char)) Curry_Prelude.C_Int
    C_CVisibility C_CTypeExpr (Curry_Prelude.OP_List C_CRule)
  | Choice_C_CFuncDecl Cover ID C_CFuncDecl C_CFuncDecl
  | Choices_C_CFuncDecl Cover ID [C_CFuncDecl]
  | Fail_C_CFuncDecl Cover FailInfo
  | Guard_C_CFuncDecl Cover Constraints C_CFuncDecl

instance Show C_CFuncDecl where
  showsPrec d (Choice_C_CFuncDecl cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CFuncDecl cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CFuncDecl cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CFuncDecl cd info) = showChar '!'
  showsPrec _ (C_CFunc x1 x2 x3 x4 x5) = showString "(CFunc" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 .
    (showChar ' ' . (shows x4 . (showChar ' ' . (shows x5 . showChar
    ')'))))))))))
  showsPrec _ (C_CmtFunc x1 x2 x3 x4 x5 x6) = showString "(CmtFunc" . (showChar
    ' ' . (shows x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 .
    (showChar ' ' . (shows x4 . (showChar ' ' . (shows x5 . (showChar ' ' .
    (shows x6 . showChar ')'))))))))))))

instance Read C_CFuncDecl where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CFunc x1 x2 x3 x4 x5, r5) | (_
    , r0) <- readQualified "AbstractCurry" "CFunc" r, (x1, r1) <- readsPrec 11
    r0, (x2, r2) <- readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2, (x4, r4) <-
    readsPrec 11 r3, (x5, r5) <- readsPrec 11 r4]) s ++ readParen ((>) d 10)
    (\r -> [(C_CmtFunc x1 x2 x3 x4 x5 x6, r6) | (_, r0) <- readQualified
    "AbstractCurry" "CmtFunc" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <-
    readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2, (x4, r4) <- readsPrec 11 r3,
    (x5, r5) <- readsPrec 11 r4, (x6, r6) <- readsPrec 11 r5]) s

instance NonDet C_CFuncDecl where
  choiceCons = Choice_C_CFuncDecl
  choicesCons = Choices_C_CFuncDecl
  failCons = Fail_C_CFuncDecl
  guardCons = Guard_C_CFuncDecl
  try (Choice_C_CFuncDecl cd i x y) = tryChoice cd i x y
  try (Choices_C_CFuncDecl cd i xs) = tryChoices cd i xs
  try (Fail_C_CFuncDecl cd info) = Fail cd info
  try (Guard_C_CFuncDecl cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CFuncDecl cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CFuncDecl cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CFuncDecl cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CFuncDecl cd i _) = error
    ("AbstractCurry.CFuncDecl.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CFuncDecl cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CFuncDecl cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CFuncDecl where
  generate s c = Choices_C_CFuncDecl c (freeID [5, 6] s) [C_CFunc (generate
    (leftSupply (leftSupply (leftSupply s))) c) (generate (rightSupply
    (leftSupply (leftSupply s))) c) (generate (rightSupply (leftSupply s)) c)
    (generate (leftSupply (rightSupply s)) c) (generate (rightSupply
    (rightSupply s)) c), C_CmtFunc (generate (leftSupply (leftSupply (leftSupply
    s))) c) (generate (rightSupply (leftSupply (leftSupply s))) c) (generate
    (rightSupply (leftSupply s)) c) (generate (leftSupply (leftSupply
    (rightSupply s))) c) (generate (rightSupply (leftSupply (rightSupply s))) c)
    (generate (rightSupply (rightSupply s)) c)]

instance NormalForm C_CFuncDecl where
  ($!!) cont (C_CFunc x1 x2 x3 x4 x5) d cs = ($!!) (\y1 d cs -> ($!!)
    (\y2 d cs -> ($!!) (\y3 d cs -> ($!!) (\y4 d cs -> ($!!) (\y5 d cs -> cont
    (C_CFunc y1 y2 y3 y4 y5) d cs) x5 d cs) x4 d cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CmtFunc x1 x2 x3 x4 x5 x6) d cs = ($!!) (\y1 d cs -> ($!!)
    (\y2 d cs -> ($!!) (\y3 d cs -> ($!!) (\y4 d cs -> ($!!) (\y5 d cs -> ($!!)
    (\y6 d cs -> cont (C_CmtFunc y1 y2 y3 y4 y5 y6) d cs) x6 d cs) x5 d cs) x4 d
    cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CFuncDecl cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CFuncDecl cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CFuncDecl cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CFuncDecl cd info) _ _ = failCons cd info
  ($##) cont (C_CFunc x1 x2 x3 x4 x5) d cs = ($##) (\y1 d cs -> ($##)
    (\y2 d cs -> ($##) (\y3 d cs -> ($##) (\y4 d cs -> ($##) (\y5 d cs -> cont
    (C_CFunc y1 y2 y3 y4 y5) d cs) x5 d cs) x4 d cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (C_CmtFunc x1 x2 x3 x4 x5 x6) d cs = ($##) (\y1 d cs -> ($##)
    (\y2 d cs -> ($##) (\y3 d cs -> ($##) (\y4 d cs -> ($##) (\y5 d cs -> ($##)
    (\y6 d cs -> cont (C_CmtFunc y1 y2 y3 y4 y5 y6) d cs) x6 d cs) x5 d cs) x4 d
    cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CFuncDecl cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CFuncDecl cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CFuncDecl cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CFuncDecl cd info) _ _ = failCons cd info
  showCons (C_CFunc _ _ _ _ _) = "AbstractCurry.CFunc _ _ _ _ _"
  showCons (C_CmtFunc _ _ _ _ _ _) = "AbstractCurry.CmtFunc _ _ _ _ _ _"
  showCons x = error ("AbstractCurry.CFuncDecl.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CFunc x1 x2 x3 x4 x5) = search (\y1 -> search (\y2 ->
    search (\y3 -> search (\y4 -> search (\y5 -> cont (C_CFunc y1 y2 y3 y4 y5))
    x5) x4) x3) x2) x1
  searchNF search cont (C_CmtFunc x1 x2 x3 x4 x5 x6) = search (\y1 -> search
    (\y2 -> search (\y3 -> search (\y4 -> search (\y5 -> search (\y6 -> cont
    (C_CmtFunc y1 y2 y3 y4 y5 y6)) x6) x5) x4) x3) x2) x1
  searchNF _ _ x = error
    ("AbstractCurry.CFuncDecl.searchNF: no constructor: " ++ show x)

instance Unifiable C_CFuncDecl where
  (=.=) (C_CFunc x1 x2 x3 x4 x5) (C_CFunc y1 y2 y3 y4 y5) d cs = (&) ((=:=) x1
    y1 d cs) ((&) ((=:=) x2 y2 d cs) ((&) ((=:=) x3 y3 d cs) ((&) ((=:=) x4 y4 d
    cs) ((=:=) x5 y5 d cs) d cs) d cs) d cs) d cs
  (=.=) (C_CmtFunc x1 x2 x3 x4 x5 x6) (C_CmtFunc y1 y2 y3 y4 y5 y6) d cs = (&)
    ((=:=) x1 y1 d cs) ((&) ((=:=) x2 y2 d cs) ((&) ((=:=) x3 y3 d cs) ((&)
    ((=:=) x4 y4 d cs) ((&) ((=:=) x5 y5 d cs) ((=:=) x6 y6 d cs) d cs) d cs) d
    cs) d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CFunc x1 x2 x3 x4 x5) (C_CFunc y1 y2 y3 y4 y5) d cs = (&) ((=:<=) x1
    y1 d cs) ((&) ((=:<=) x2 y2 d cs) ((&) ((=:<=) x3 y3 d cs) ((&) ((=:<=) x4
    y4 d cs) ((=:<=) x5 y5 d cs) d cs) d cs) d cs) d cs
  (=.<=) (C_CmtFunc x1 x2 x3 x4 x5 x6) (C_CmtFunc y1 y2 y3 y4 y5 y6) d cs = (&)
    ((=:<=) x1 y1 d cs) ((&) ((=:<=) x2 y2 d cs) ((&) ((=:<=) x3 y3 d cs) ((&)
    ((=:<=) x4 y4 d cs) ((&) ((=:<=) x5 y5 d cs) ((=:<=) x6 y6 d cs) d cs) d cs)
    d cs) d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CFunc x3 x4 x5 x6 x7) = (i :=: ChooseN 0 5) : concat [bind cd
    (leftID (leftID (leftID i))) x3, bind cd (rightID (leftID (leftID i))) x4
    , bind cd (rightID (leftID i)) x5, bind cd (leftID (rightID i)) x6, bind cd
    (rightID (rightID i)) x7]
  bind cd i (C_CmtFunc x3 x4 x5 x6 x7 x8) = (i :=: ChooseN 1 6) : concat [bind
    cd (leftID (leftID (leftID i))) x3, bind cd (rightID (leftID (leftID i))) x4
    , bind cd (rightID (leftID i)) x5, bind cd (leftID (leftID (rightID i))) x6
    , bind cd (rightID (leftID (rightID i))) x7, bind cd (rightID (rightID i))
    x8]
  bind d i (Choice_C_CFuncDecl cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CFuncDecl cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CFuncDecl cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CFuncDecl cd i _) = error
    ("AbstractCurry.CFuncDecl.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CFuncDecl cd info) = [Unsolvable info]
  bind d i (Guard_C_CFuncDecl cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CFunc x3 x4 x5 x6 x7) = (i :=: ChooseN 0 5) : [leftID (leftID
    (leftID i)) :=: LazyBind (lazyBind cd (leftID (leftID (leftID i))) x3)
    , rightID (leftID (leftID i)) :=: LazyBind (lazyBind cd (rightID (leftID
    (leftID i))) x4), rightID (leftID i) :=: LazyBind (lazyBind cd (rightID
    (leftID i)) x5), leftID (rightID i) :=: LazyBind (lazyBind cd (leftID
    (rightID i)) x6), rightID (rightID i) :=: LazyBind (lazyBind cd (rightID
    (rightID i)) x7)]
  lazyBind cd i (C_CmtFunc x3 x4 x5 x6 x7 x8) = (i :=: ChooseN 1 6) : [leftID
    (leftID (leftID i)) :=: LazyBind (lazyBind cd (leftID (leftID (leftID i)))
    x3), rightID (leftID (leftID i)) :=: LazyBind (lazyBind cd (rightID (leftID
    (leftID i))) x4), rightID (leftID i) :=: LazyBind (lazyBind cd (rightID
    (leftID i)) x5), leftID (leftID (rightID i)) :=: LazyBind (lazyBind cd
    (leftID (leftID (rightID i))) x6), rightID (leftID (rightID i)) :=: LazyBind
    (lazyBind cd (rightID (leftID (rightID i))) x7), rightID (rightID i) :=:
    LazyBind (lazyBind cd (rightID (rightID i)) x8)]
  lazyBind d i (Choice_C_CFuncDecl cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CFuncDecl cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CFuncDecl cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CFuncDecl cd i _) = error
    ("AbstractCurry.CFuncDecl.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CFuncDecl cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CFuncDecl cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CFuncDecl where
  (=?=) (Choice_C_CFuncDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CFuncDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CFuncDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CFuncDecl cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CFuncDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CFuncDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CFuncDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CFuncDecl cd info) _ _ = failCons cd info
  (=?=) (C_CFunc x1 x2 x3 x4 x5) (C_CFunc y1 y2 y3 y4 y5) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x4 y4 d cs)
    ((Curry_Prelude.=?=) x5 y5 d cs) d cs) d cs) d cs) d cs
  (=?=) (C_CmtFunc x1 x2 x3 x4 x5 x6) (C_CmtFunc y1 y2 y3 y4 y5 y6) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x4 y4 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x5 y5 d cs)
    ((Curry_Prelude.=?=) x6 y6 d cs) d cs) d cs) d cs) d cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CFuncDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CFuncDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CFuncDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CFuncDecl cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CFuncDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CFuncDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CFuncDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CFuncDecl cd info) _ _ = failCons cd info
  (<?=) (C_CFunc x1 x2 x3 x4 x5) (C_CFunc y1 y2 y3 y4 y5) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x4 y4 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x4 y4 d cs)
    ((Curry_Prelude.<?=) x5 y5 d cs) d cs) d cs) d cs) d cs) d cs) d cs) d cs) d
    cs
  (<?=) (C_CFunc _ _ _ _ _) (C_CmtFunc _ _ _ _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CmtFunc x1 x2 x3 x4 x5 x6) (C_CmtFunc y1 y2 y3 y4 y5 y6) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x2 y2 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x2 y2 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x3 y3 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x3 y3 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x4 y4 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x4 y4 d cs)
    (Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x5 y5 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x5 y5 d cs)
    ((Curry_Prelude.<?=) x6 y6 d cs) d cs) d cs) d cs) d cs) d cs) d cs) d cs) d
    cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CRule
  = C_CRule (Curry_Prelude.OP_List C_CPattern) C_CRhs
  | Choice_C_CRule Cover ID C_CRule C_CRule
  | Choices_C_CRule Cover ID [C_CRule]
  | Fail_C_CRule Cover FailInfo
  | Guard_C_CRule Cover Constraints C_CRule

instance Show C_CRule where
  showsPrec d (Choice_C_CRule cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CRule cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CRule cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CRule cd info) = showChar '!'
  showsPrec _ (C_CRule x1 x2) = showString "(CRule" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . showChar ')'))))

instance Read C_CRule where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CRule x1 x2, r2) | (_, r0) <-
    readQualified "AbstractCurry" "CRule" r, (x1, r1) <- readsPrec 11 r0, (x2
    , r2) <- readsPrec 11 r1]) s

instance NonDet C_CRule where
  choiceCons = Choice_C_CRule
  choicesCons = Choices_C_CRule
  failCons = Fail_C_CRule
  guardCons = Guard_C_CRule
  try (Choice_C_CRule cd i x y) = tryChoice cd i x y
  try (Choices_C_CRule cd i xs) = tryChoices cd i xs
  try (Fail_C_CRule cd info) = Fail cd info
  try (Guard_C_CRule cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CRule cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CRule cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CRule cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CRule cd i _) = error
    ("AbstractCurry.CRule.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CRule cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CRule cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CRule where
  generate s c = Choices_C_CRule c (freeID [2] s) [C_CRule (generate (leftSupply
    s) c) (generate (rightSupply s) c)]

instance NormalForm C_CRule where
  ($!!) cont (C_CRule x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> cont
    (C_CRule y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CRule cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CRule cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CRule cd c e) d cs = guardCons cd c (($!!) cont e d (addCs
    c cs))
  ($!!) _ (Fail_C_CRule cd info) _ _ = failCons cd info
  ($##) cont (C_CRule x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> cont
    (C_CRule y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CRule cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CRule cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CRule cd c e) d cs = guardCons cd c (($##) cont e d (addCs
    c cs))
  ($##) _ (Fail_C_CRule cd info) _ _ = failCons cd info
  showCons (C_CRule _ _) = "AbstractCurry.CRule _ _"
  showCons x = error ("AbstractCurry.CRule.showCons: no constructor: " ++ show
    x)
  searchNF search cont (C_CRule x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CRule y1 y2)) x2) x1
  searchNF _ _ x = error ("AbstractCurry.CRule.searchNF: no constructor: " ++
    show x)

instance Unifiable C_CRule where
  (=.=) (C_CRule x1 x2) (C_CRule y1 y2) d cs = (&) ((=:=) x1 y1 d cs) ((=:=) x2
    y2 d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CRule x1 x2) (C_CRule y1 y2) d cs = (&) ((=:<=) x1 y1 d cs) ((=:<=)
    x2 y2 d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CRule x3 x4) = (i :=: ChooseN 0 2) : concat [bind cd (leftID i)
    x3, bind cd (rightID i) x4]
  bind d i (Choice_C_CRule cd j x y) = [ConstraintChoice cd j (bind d i x) (bind
    d i y)]
  bind d i (Choices_C_CRule cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CRule cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd j
    (map (bind d i) xs)]
  bind _ _ (Choices_C_CRule cd i _) = error
    ("AbstractCurry.CRule.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CRule cd info) = [Unsolvable info]
  bind d i (Guard_C_CRule cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CRule x3 x4) = (i :=: ChooseN 0 2) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd (rightID i)
    x4)]
  lazyBind d i (Choice_C_CRule cd j x y) = [ConstraintChoice cd j (lazyBind d i
    x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CRule cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i cd
    j xs
  lazyBind d i (Choices_C_CRule cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CRule cd i _) = error
    ("AbstractCurry.CRule.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CRule cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CRule cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CRule where
  (=?=) (Choice_C_CRule cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=) x z
    d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CRule cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CRule cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=) e y
    d (addCs c cs))
  (=?=) (Fail_C_CRule cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CRule cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=) z x
    d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CRule cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CRule cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=) y e
    d (addCs c cs))
  (=?=) _ (Fail_C_CRule cd info) _ _ = failCons cd info
  (=?=) (C_CRule x1 x2) (C_CRule y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (<?=) (Choice_C_CRule cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=) x z
    d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CRule cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CRule cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=) e y
    d (addCs c cs))
  (<?=) (Fail_C_CRule cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CRule cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=) z x
    d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CRule cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CRule cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=) y e
    d (addCs c cs))
  (<?=) _ (Fail_C_CRule cd info) _ _ = failCons cd info
  (<?=) (C_CRule x1 x2) (C_CRule y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs

data C_CRhs
  = C_CSimpleRhs C_CExpr (Curry_Prelude.OP_List C_CLocalDecl)
  | C_CGuardedRhs (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_CExpr
    C_CExpr)) (Curry_Prelude.OP_List C_CLocalDecl)
  | Choice_C_CRhs Cover ID C_CRhs C_CRhs
  | Choices_C_CRhs Cover ID [C_CRhs]
  | Fail_C_CRhs Cover FailInfo
  | Guard_C_CRhs Cover Constraints C_CRhs

instance Show C_CRhs where
  showsPrec d (Choice_C_CRhs cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CRhs cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CRhs cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CRhs cd info) = showChar '!'
  showsPrec _ (C_CSimpleRhs x1 x2) = showString "(CSimpleRhs" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CGuardedRhs x1 x2) = showString "(CGuardedRhs" . (showChar
    ' ' . (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))

instance Read C_CRhs where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CSimpleRhs x1 x2, r2) | (_
    , r0) <- readQualified "AbstractCurry" "CSimpleRhs" r, (x1, r1) <- readsPrec
    11 r0, (x2, r2) <- readsPrec 11 r1]) s ++ readParen ((>) d 10) (\r ->
    [(C_CGuardedRhs x1 x2, r2) | (_, r0) <- readQualified "AbstractCurry"
    "CGuardedRhs" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1])
    s

instance NonDet C_CRhs where
  choiceCons = Choice_C_CRhs
  choicesCons = Choices_C_CRhs
  failCons = Fail_C_CRhs
  guardCons = Guard_C_CRhs
  try (Choice_C_CRhs cd i x y) = tryChoice cd i x y
  try (Choices_C_CRhs cd i xs) = tryChoices cd i xs
  try (Fail_C_CRhs cd info) = Fail cd info
  try (Guard_C_CRhs cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CRhs cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CRhs cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CRhs cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CRhs cd i _) = error
    ("AbstractCurry.CRhs.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CRhs cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CRhs cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CRhs where
  generate s c = Choices_C_CRhs c (freeID [2, 2] s) [C_CSimpleRhs (generate
    (leftSupply s) c) (generate (rightSupply s) c), C_CGuardedRhs (generate
    (leftSupply s) c) (generate (rightSupply s) c)]

instance NormalForm C_CRhs where
  ($!!) cont (C_CSimpleRhs x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CSimpleRhs y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CGuardedRhs x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CGuardedRhs y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CRhs cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CRhs cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CRhs cd c e) d cs = guardCons cd c (($!!) cont e d (addCs
    c cs))
  ($!!) _ (Fail_C_CRhs cd info) _ _ = failCons cd info
  ($##) cont (C_CSimpleRhs x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CSimpleRhs y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CGuardedRhs x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CGuardedRhs y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CRhs cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CRhs cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CRhs cd c e) d cs = guardCons cd c (($##) cont e d (addCs
    c cs))
  ($##) _ (Fail_C_CRhs cd info) _ _ = failCons cd info
  showCons (C_CSimpleRhs _ _) = "AbstractCurry.CSimpleRhs _ _"
  showCons (C_CGuardedRhs _ _) = "AbstractCurry.CGuardedRhs _ _"
  showCons x = error ("AbstractCurry.CRhs.showCons: no constructor: " ++ show x)
  searchNF search cont (C_CSimpleRhs x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CSimpleRhs y1 y2)) x2) x1
  searchNF search cont (C_CGuardedRhs x1 x2) = search (\y1 -> search (\y2 ->
    cont (C_CGuardedRhs y1 y2)) x2) x1
  searchNF _ _ x = error ("AbstractCurry.CRhs.searchNF: no constructor: " ++
    show x)

instance Unifiable C_CRhs where
  (=.=) (C_CSimpleRhs x1 x2) (C_CSimpleRhs y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) (C_CGuardedRhs x1 x2) (C_CGuardedRhs y1 y2) d cs = (&) ((=:=) x1 y1 d
    cs) ((=:=) x2 y2 d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CSimpleRhs x1 x2) (C_CSimpleRhs y1 y2) d cs = (&) ((=:<=) x1 y1 d
    cs) ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CGuardedRhs x1 x2) (C_CGuardedRhs y1 y2) d cs = (&) ((=:<=) x1 y1 d
    cs) ((=:<=) x2 y2 d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CSimpleRhs x3 x4) = (i :=: ChooseN 0 2) : concat [bind cd (leftID
    i) x3, bind cd (rightID i) x4]
  bind cd i (C_CGuardedRhs x3 x4) = (i :=: ChooseN 1 2) : concat [bind cd
    (leftID i) x3, bind cd (rightID i) x4]
  bind d i (Choice_C_CRhs cd j x y) = [ConstraintChoice cd j (bind d i x) (bind
    d i y)]
  bind d i (Choices_C_CRhs cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CRhs cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd j
    (map (bind d i) xs)]
  bind _ _ (Choices_C_CRhs cd i _) = error
    ("AbstractCurry.CRhs.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CRhs cd info) = [Unsolvable info]
  bind d i (Guard_C_CRhs cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CSimpleRhs x3 x4) = (i :=: ChooseN 0 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind cd i (C_CGuardedRhs x3 x4) = (i :=: ChooseN 1 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind d i (Choice_C_CRhs cd j x y) = [ConstraintChoice cd j (lazyBind d i
    x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CRhs cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i cd j
    xs
  lazyBind d i (Choices_C_CRhs cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd
    j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CRhs cd i _) = error
    ("AbstractCurry.CRhs.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CRhs cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CRhs cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CRhs where
  (=?=) (Choice_C_CRhs cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=) x z d
    cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CRhs cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CRhs cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=) e y d
    (addCs c cs))
  (=?=) (Fail_C_CRhs cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CRhs cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=) z x d
    cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CRhs cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CRhs cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=) y e d
    (addCs c cs))
  (=?=) _ (Fail_C_CRhs cd info) _ _ = failCons cd info
  (=?=) (C_CSimpleRhs x1 x2) (C_CSimpleRhs y1 y2) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CGuardedRhs x1 x2) (C_CGuardedRhs y1 y2) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CRhs cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=) x z d
    cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CRhs cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CRhs cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=) e y d
    (addCs c cs))
  (<?=) (Fail_C_CRhs cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CRhs cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=) z x d
    cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CRhs cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CRhs cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=) y e d
    (addCs c cs))
  (<?=) _ (Fail_C_CRhs cd info) _ _ = failCons cd info
  (<?=) (C_CSimpleRhs x1 x2) (C_CSimpleRhs y1 y2) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CSimpleRhs _ _) (C_CGuardedRhs _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CGuardedRhs x1 x2) (C_CGuardedRhs y1 y2) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CLocalDecl
  = C_CLocalFunc C_CFuncDecl
  | C_CLocalPat C_CPattern C_CRhs
  | C_CLocalVars (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2
    Curry_Prelude.C_Int (Curry_Prelude.OP_List Curry_Prelude.C_Char)))
  | Choice_C_CLocalDecl Cover ID C_CLocalDecl C_CLocalDecl
  | Choices_C_CLocalDecl Cover ID [C_CLocalDecl]
  | Fail_C_CLocalDecl Cover FailInfo
  | Guard_C_CLocalDecl Cover Constraints C_CLocalDecl

instance Show C_CLocalDecl where
  showsPrec d (Choice_C_CLocalDecl cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CLocalDecl cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CLocalDecl cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CLocalDecl cd info) = showChar '!'
  showsPrec _ (C_CLocalFunc x1) = showString "(CLocalFunc" . (showChar ' ' .
    (shows x1 . showChar ')'))
  showsPrec _ (C_CLocalPat x1 x2) = showString "(CLocalPat" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CLocalVars x1) = showString "(CLocalVars" . (showChar ' ' .
    (shows x1 . showChar ')'))

instance Read C_CLocalDecl where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CLocalFunc x1, r1) | (_, r0)
    <- readQualified "AbstractCurry" "CLocalFunc" r, (x1, r1) <- readsPrec 11
    r0]) s ++ (readParen ((>) d 10) (\r -> [(C_CLocalPat x1 x2, r2) | (_, r0) <-
    readQualified "AbstractCurry" "CLocalPat" r, (x1, r1) <- readsPrec 11 r0,
    (x2, r2) <- readsPrec 11 r1]) s ++ readParen ((>) d 10) (\r ->
    [(C_CLocalVars x1, r1) | (_, r0) <- readQualified "AbstractCurry"
    "CLocalVars" r, (x1, r1) <- readsPrec 11 r0]) s)

instance NonDet C_CLocalDecl where
  choiceCons = Choice_C_CLocalDecl
  choicesCons = Choices_C_CLocalDecl
  failCons = Fail_C_CLocalDecl
  guardCons = Guard_C_CLocalDecl
  try (Choice_C_CLocalDecl cd i x y) = tryChoice cd i x y
  try (Choices_C_CLocalDecl cd i xs) = tryChoices cd i xs
  try (Fail_C_CLocalDecl cd info) = Fail cd info
  try (Guard_C_CLocalDecl cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CLocalDecl cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CLocalDecl cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CLocalDecl cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CLocalDecl cd i _) = error
    ("AbstractCurry.CLocalDecl.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CLocalDecl cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CLocalDecl cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CLocalDecl where
  generate s c = Choices_C_CLocalDecl c (freeID [1, 2, 1] s) [C_CLocalFunc
    (generate (leftSupply s) c), C_CLocalPat (generate (leftSupply s) c)
    (generate (rightSupply s) c), C_CLocalVars (generate (leftSupply s) c)]

instance NormalForm C_CLocalDecl where
  ($!!) cont (C_CLocalFunc x1) d cs = ($!!) (\y1 d cs -> cont (C_CLocalFunc y1)
    d cs) x1 d cs
  ($!!) cont (C_CLocalPat x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CLocalPat y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CLocalVars x1) d cs = ($!!) (\y1 d cs -> cont (C_CLocalVars y1)
    d cs) x1 d cs
  ($!!) cont (Choice_C_CLocalDecl cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CLocalDecl cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CLocalDecl cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CLocalDecl cd info) _ _ = failCons cd info
  ($##) cont (C_CLocalFunc x1) d cs = ($##) (\y1 d cs -> cont (C_CLocalFunc y1)
    d cs) x1 d cs
  ($##) cont (C_CLocalPat x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CLocalPat y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CLocalVars x1) d cs = ($##) (\y1 d cs -> cont (C_CLocalVars y1)
    d cs) x1 d cs
  ($##) cont (Choice_C_CLocalDecl cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CLocalDecl cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CLocalDecl cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CLocalDecl cd info) _ _ = failCons cd info
  showCons (C_CLocalFunc _) = "AbstractCurry.CLocalFunc _"
  showCons (C_CLocalPat _ _) = "AbstractCurry.CLocalPat _ _"
  showCons (C_CLocalVars _) = "AbstractCurry.CLocalVars _"
  showCons x = error ("AbstractCurry.CLocalDecl.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CLocalFunc x1) = search (\y1 -> cont (C_CLocalFunc
    y1)) x1
  searchNF search cont (C_CLocalPat x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CLocalPat y1 y2)) x2) x1
  searchNF search cont (C_CLocalVars x1) = search (\y1 -> cont (C_CLocalVars
    y1)) x1
  searchNF _ _ x = error
    ("AbstractCurry.CLocalDecl.searchNF: no constructor: " ++ show x)

instance Unifiable C_CLocalDecl where
  (=.=) (C_CLocalFunc x1) (C_CLocalFunc y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CLocalPat x1 x2) (C_CLocalPat y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) (C_CLocalVars x1) (C_CLocalVars y1) d cs = (=:=) x1 y1 d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CLocalFunc x1) (C_CLocalFunc y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CLocalPat x1 x2) (C_CLocalPat y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CLocalVars x1) (C_CLocalVars y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CLocalFunc x3) = (i :=: ChooseN 0 1) : concat [bind cd (leftID i)
    x3]
  bind cd i (C_CLocalPat x3 x4) = (i :=: ChooseN 1 2) : concat [bind cd (leftID
    i) x3, bind cd (rightID i) x4]
  bind cd i (C_CLocalVars x3) = (i :=: ChooseN 2 1) : concat [bind cd (leftID i)
    x3]
  bind d i (Choice_C_CLocalDecl cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CLocalDecl cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j
    xs
  bind d i (Choices_C_CLocalDecl cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CLocalDecl cd i _) = error
    ("AbstractCurry.CLocalDecl.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CLocalDecl cd info) = [Unsolvable info]
  bind d i (Guard_C_CLocalDecl cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CLocalFunc x3) = (i :=: ChooseN 0 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CLocalPat x3 x4) = (i :=: ChooseN 1 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind cd i (C_CLocalVars x3) = (i :=: ChooseN 2 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind d i (Choice_C_CLocalDecl cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CLocalDecl cd j@(FreeID _ _) xs) = lazyBindOrNarrow d
    i cd j xs
  lazyBind d i (Choices_C_CLocalDecl cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CLocalDecl cd i _) = error
    ("AbstractCurry.CLocalDecl.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CLocalDecl cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CLocalDecl cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CLocalDecl where
  (=?=) (Choice_C_CLocalDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CLocalDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CLocalDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CLocalDecl cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CLocalDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CLocalDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CLocalDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CLocalDecl cd info) _ _ = failCons cd info
  (=?=) (C_CLocalFunc x1) (C_CLocalFunc y1) d cs = (Curry_Prelude.=?=) x1 y1 d
    cs
  (=?=) (C_CLocalPat x1 x2) (C_CLocalPat y1 y2) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CLocalVars x1) (C_CLocalVars y1) d cs = (Curry_Prelude.=?=) x1 y1 d
    cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CLocalDecl cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CLocalDecl cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CLocalDecl cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CLocalDecl cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CLocalDecl cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CLocalDecl cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CLocalDecl cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CLocalDecl cd info) _ _ = failCons cd info
  (<?=) (C_CLocalFunc x1) (C_CLocalFunc y1) d cs = (Curry_Prelude.<?=) x1 y1 d
    cs
  (<?=) (C_CLocalFunc _) (C_CLocalPat _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLocalFunc _) (C_CLocalVars _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLocalPat x1 x2) (C_CLocalPat y1 y2) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CLocalPat _ _) (C_CLocalVars _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLocalVars x1) (C_CLocalVars y1) d cs = (Curry_Prelude.<?=) x1 y1 d
    cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CPattern
  = C_CPVar (Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List
    Curry_Prelude.C_Char))
  | C_CPLit C_CLiteral
  | C_CPComb (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    (Curry_Prelude.OP_List C_CPattern)
  | C_CPAs (Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List
    Curry_Prelude.C_Char)) C_CPattern
  | C_CPFuncComb (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    (Curry_Prelude.OP_List C_CPattern)
  | C_CPLazy C_CPattern
  | C_CPRecord (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_Tuple2
    (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List
    Curry_Prelude.C_Char)) C_CPattern))
  | Choice_C_CPattern Cover ID C_CPattern C_CPattern
  | Choices_C_CPattern Cover ID [C_CPattern]
  | Fail_C_CPattern Cover FailInfo
  | Guard_C_CPattern Cover Constraints C_CPattern

instance Show C_CPattern where
  showsPrec d (Choice_C_CPattern cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CPattern cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CPattern cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CPattern cd info) = showChar '!'
  showsPrec _ (C_CPVar x1) = showString "(CPVar" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CPLit x1) = showString "(CPLit" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CPComb x1 x2) = showString "(CPComb" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CPAs x1 x2) = showString "(CPAs" . (showChar ' ' . (shows x1 .
    (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CPFuncComb x1 x2) = showString "(CPFuncComb" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CPLazy x1) = showString "(CPLazy" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CPRecord x1 x2) = showString "(CPRecord" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))

instance Read C_CPattern where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CPVar x1, r1) | (_, r0) <-
    readQualified "AbstractCurry" "CPVar" r, (x1, r1) <- readsPrec 11 r0]) s ++
    (readParen ((>) d 10) (\r -> [(C_CPLit x1, r1) | (_, r0) <- readQualified
    "AbstractCurry" "CPLit" r, (x1, r1) <- readsPrec 11 r0]) s ++ (readParen
    ((>) d 10) (\r -> [(C_CPComb x1 x2, r2) | (_, r0) <- readQualified
    "AbstractCurry" "CPComb" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <-
    readsPrec 11 r1]) s ++ (readParen ((>) d 10) (\r -> [(C_CPAs x1 x2, r2) | (_
    , r0) <- readQualified "AbstractCurry" "CPAs" r, (x1, r1) <- readsPrec 11
    r0, (x2, r2) <- readsPrec 11 r1]) s ++ (readParen ((>) d 10) (\r ->
    [(C_CPFuncComb x1 x2, r2) | (_, r0) <- readQualified "AbstractCurry"
    "CPFuncComb" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1])
    s ++ (readParen ((>) d 10) (\r -> [(C_CPLazy x1, r1) | (_, r0) <-
    readQualified "AbstractCurry" "CPLazy" r, (x1, r1) <- readsPrec 11 r0]) s ++
    readParen ((>) d 10) (\r -> [(C_CPRecord x1 x2, r2) | (_, r0) <-
    readQualified "AbstractCurry" "CPRecord" r, (x1, r1) <- readsPrec 11 r0, (x2
    , r2) <- readsPrec 11 r1]) s)))))

instance NonDet C_CPattern where
  choiceCons = Choice_C_CPattern
  choicesCons = Choices_C_CPattern
  failCons = Fail_C_CPattern
  guardCons = Guard_C_CPattern
  try (Choice_C_CPattern cd i x y) = tryChoice cd i x y
  try (Choices_C_CPattern cd i xs) = tryChoices cd i xs
  try (Fail_C_CPattern cd info) = Fail cd info
  try (Guard_C_CPattern cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CPattern cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CPattern cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CPattern cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CPattern cd i _) = error
    ("AbstractCurry.CPattern.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CPattern cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CPattern cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CPattern where
  generate s c = Choices_C_CPattern c (freeID [1, 1, 2, 2, 2, 1, 2] s) [C_CPVar
    (generate (leftSupply s) c), C_CPLit (generate (leftSupply s) c), C_CPComb
    (generate (leftSupply s) c) (generate (rightSupply s) c), C_CPAs (generate
    (leftSupply s) c) (generate (rightSupply s) c), C_CPFuncComb (generate
    (leftSupply s) c) (generate (rightSupply s) c), C_CPLazy (generate
    (leftSupply s) c), C_CPRecord (generate (leftSupply s) c) (generate
    (rightSupply s) c)]

instance NormalForm C_CPattern where
  ($!!) cont (C_CPVar x1) d cs = ($!!) (\y1 d cs -> cont (C_CPVar y1) d cs) x1 d
    cs
  ($!!) cont (C_CPLit x1) d cs = ($!!) (\y1 d cs -> cont (C_CPLit y1) d cs) x1 d
    cs
  ($!!) cont (C_CPComb x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> cont
    (C_CPComb y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CPAs x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> cont
    (C_CPAs y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CPFuncComb x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CPFuncComb y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CPLazy x1) d cs = ($!!) (\y1 d cs -> cont (C_CPLazy y1) d cs) x1
    d cs
  ($!!) cont (C_CPRecord x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CPRecord y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CPattern cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CPattern cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CPattern cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CPattern cd info) _ _ = failCons cd info
  ($##) cont (C_CPVar x1) d cs = ($##) (\y1 d cs -> cont (C_CPVar y1) d cs) x1 d
    cs
  ($##) cont (C_CPLit x1) d cs = ($##) (\y1 d cs -> cont (C_CPLit y1) d cs) x1 d
    cs
  ($##) cont (C_CPComb x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> cont
    (C_CPComb y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CPAs x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> cont
    (C_CPAs y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CPFuncComb x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CPFuncComb y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CPLazy x1) d cs = ($##) (\y1 d cs -> cont (C_CPLazy y1) d cs) x1
    d cs
  ($##) cont (C_CPRecord x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CPRecord y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CPattern cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CPattern cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CPattern cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CPattern cd info) _ _ = failCons cd info
  showCons (C_CPVar _) = "AbstractCurry.CPVar _"
  showCons (C_CPLit _) = "AbstractCurry.CPLit _"
  showCons (C_CPComb _ _) = "AbstractCurry.CPComb _ _"
  showCons (C_CPAs _ _) = "AbstractCurry.CPAs _ _"
  showCons (C_CPFuncComb _ _) = "AbstractCurry.CPFuncComb _ _"
  showCons (C_CPLazy _) = "AbstractCurry.CPLazy _"
  showCons (C_CPRecord _ _) = "AbstractCurry.CPRecord _ _"
  showCons x = error ("AbstractCurry.CPattern.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CPVar x1) = search (\y1 -> cont (C_CPVar y1)) x1
  searchNF search cont (C_CPLit x1) = search (\y1 -> cont (C_CPLit y1)) x1
  searchNF search cont (C_CPComb x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CPComb y1 y2)) x2) x1
  searchNF search cont (C_CPAs x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CPAs y1 y2)) x2) x1
  searchNF search cont (C_CPFuncComb x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CPFuncComb y1 y2)) x2) x1
  searchNF search cont (C_CPLazy x1) = search (\y1 -> cont (C_CPLazy y1)) x1
  searchNF search cont (C_CPRecord x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CPRecord y1 y2)) x2) x1
  searchNF _ _ x = error ("AbstractCurry.CPattern.searchNF: no constructor: " ++
    show x)

instance Unifiable C_CPattern where
  (=.=) (C_CPVar x1) (C_CPVar y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CPLit x1) (C_CPLit y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CPComb x1 x2) (C_CPComb y1 y2) d cs = (&) ((=:=) x1 y1 d cs) ((=:=)
    x2 y2 d cs) d cs
  (=.=) (C_CPAs x1 x2) (C_CPAs y1 y2) d cs = (&) ((=:=) x1 y1 d cs) ((=:=) x2 y2
    d cs) d cs
  (=.=) (C_CPFuncComb x1 x2) (C_CPFuncComb y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) (C_CPLazy x1) (C_CPLazy y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CPRecord x1 x2) (C_CPRecord y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CPVar x1) (C_CPVar y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CPLit x1) (C_CPLit y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CPComb x1 x2) (C_CPComb y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CPAs x1 x2) (C_CPAs y1 y2) d cs = (&) ((=:<=) x1 y1 d cs) ((=:<=) x2
    y2 d cs) d cs
  (=.<=) (C_CPFuncComb x1 x2) (C_CPFuncComb y1 y2) d cs = (&) ((=:<=) x1 y1 d
    cs) ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CPLazy x1) (C_CPLazy y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CPRecord x1 x2) (C_CPRecord y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CPVar x3) = (i :=: ChooseN 0 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CPLit x3) = (i :=: ChooseN 1 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CPComb x3 x4) = (i :=: ChooseN 2 2) : concat [bind cd (leftID i)
    x3, bind cd (rightID i) x4]
  bind cd i (C_CPAs x3 x4) = (i :=: ChooseN 3 2) : concat [bind cd (leftID i) x3
    , bind cd (rightID i) x4]
  bind cd i (C_CPFuncComb x3 x4) = (i :=: ChooseN 4 2) : concat [bind cd (leftID
    i) x3, bind cd (rightID i) x4]
  bind cd i (C_CPLazy x3) = (i :=: ChooseN 5 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CPRecord x3 x4) = (i :=: ChooseN 6 2) : concat [bind cd (leftID
    i) x3, bind cd (rightID i) x4]
  bind d i (Choice_C_CPattern cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CPattern cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CPattern cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd
    j (map (bind d i) xs)]
  bind _ _ (Choices_C_CPattern cd i _) = error
    ("AbstractCurry.CPattern.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CPattern cd info) = [Unsolvable info]
  bind d i (Guard_C_CPattern cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CPVar x3) = (i :=: ChooseN 0 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CPLit x3) = (i :=: ChooseN 1 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CPComb x3 x4) = (i :=: ChooseN 2 2) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd (rightID i)
    x4)]
  lazyBind cd i (C_CPAs x3 x4) = (i :=: ChooseN 3 2) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd (rightID i)
    x4)]
  lazyBind cd i (C_CPFuncComb x3 x4) = (i :=: ChooseN 4 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind cd i (C_CPLazy x3) = (i :=: ChooseN 5 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CPRecord x3 x4) = (i :=: ChooseN 6 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind d i (Choice_C_CPattern cd j x y) = [ConstraintChoice cd j (lazyBind d
    i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CPattern cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CPattern cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CPattern cd i _) = error
    ("AbstractCurry.CPattern.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CPattern cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CPattern cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CPattern where
  (=?=) (Choice_C_CPattern cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=) x
    z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CPattern cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CPattern cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=) e
    y d (addCs c cs))
  (=?=) (Fail_C_CPattern cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CPattern cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=) z
    x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CPattern cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CPattern cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=) y
    e d (addCs c cs))
  (=?=) _ (Fail_C_CPattern cd info) _ _ = failCons cd info
  (=?=) (C_CPVar x1) (C_CPVar y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CPLit x1) (C_CPLit y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CPComb x1 x2) (C_CPComb y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CPAs x1 x2) (C_CPAs y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CPFuncComb x1 x2) (C_CPFuncComb y1 y2) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CPLazy x1) (C_CPLazy y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CPRecord x1 x2) (C_CPRecord y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CPattern cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=) x
    z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CPattern cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CPattern cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=) e
    y d (addCs c cs))
  (<?=) (Fail_C_CPattern cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CPattern cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=) z
    x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CPattern cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CPattern cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=) y
    e d (addCs c cs))
  (<?=) _ (Fail_C_CPattern cd info) _ _ = failCons cd info
  (<?=) (C_CPVar x1) (C_CPVar y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CPVar _) (C_CPLit _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPVar _) (C_CPComb _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPVar _) (C_CPAs _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPVar _) (C_CPFuncComb _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPVar _) (C_CPLazy _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPVar _) (C_CPRecord _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPLit x1) (C_CPLit y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CPLit _) (C_CPComb _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPLit _) (C_CPAs _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPLit _) (C_CPFuncComb _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPLit _) (C_CPLazy _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPLit _) (C_CPRecord _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPComb x1 x2) (C_CPComb y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CPComb _ _) (C_CPAs _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPComb _ _) (C_CPFuncComb _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPComb _ _) (C_CPLazy _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPComb _ _) (C_CPRecord _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPAs x1 x2) (C_CPAs y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CPAs _ _) (C_CPFuncComb _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPAs _ _) (C_CPLazy _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPAs _ _) (C_CPRecord _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPFuncComb x1 x2) (C_CPFuncComb y1 y2) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CPFuncComb _ _) (C_CPLazy _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPFuncComb _ _) (C_CPRecord _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPLazy x1) (C_CPLazy y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CPLazy _) (C_CPRecord _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CPRecord x1 x2) (C_CPRecord y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CExpr
  = C_CVar (Curry_Prelude.OP_Tuple2 Curry_Prelude.C_Int (Curry_Prelude.OP_List
    Curry_Prelude.C_Char))
  | C_CLit C_CLiteral
  | C_CSymbol (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
  | C_CApply C_CExpr C_CExpr
  | C_CLambda (Curry_Prelude.OP_List C_CPattern) C_CExpr
  | C_CLetDecl (Curry_Prelude.OP_List C_CLocalDecl) C_CExpr
  | C_CDoExpr (Curry_Prelude.OP_List C_CStatement)
  | C_CListComp C_CExpr (Curry_Prelude.OP_List C_CStatement)
  | C_CCase C_CCaseType C_CExpr (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2
    C_CPattern C_CRhs))
  | C_CTyped C_CExpr C_CTypeExpr
  | C_CRecConstr (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List
    Curry_Prelude.C_Char) (Curry_Prelude.OP_List Curry_Prelude.C_Char))
    (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_Tuple2
    (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List
    Curry_Prelude.C_Char)) C_CExpr))
  | C_CRecUpdate C_CExpr (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2
    (Curry_Prelude.OP_Tuple2 (Curry_Prelude.OP_List Curry_Prelude.C_Char)
    (Curry_Prelude.OP_List Curry_Prelude.C_Char)) C_CExpr))
  | Choice_C_CExpr Cover ID C_CExpr C_CExpr
  | Choices_C_CExpr Cover ID [C_CExpr]
  | Fail_C_CExpr Cover FailInfo
  | Guard_C_CExpr Cover Constraints C_CExpr

instance Show C_CExpr where
  showsPrec d (Choice_C_CExpr cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CExpr cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CExpr cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CExpr cd info) = showChar '!'
  showsPrec _ (C_CVar x1) = showString "(CVar" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CLit x1) = showString "(CLit" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CSymbol x1) = showString "(CSymbol" . (showChar ' ' . (shows
    x1 . showChar ')'))
  showsPrec _ (C_CApply x1 x2) = showString "(CApply" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CLambda x1 x2) = showString "(CLambda" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CLetDecl x1 x2) = showString "(CLetDecl" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CDoExpr x1) = showString "(CDoExpr" . (showChar ' ' . (shows
    x1 . showChar ')'))
  showsPrec _ (C_CListComp x1 x2) = showString "(CListComp" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CCase x1 x2 x3) = showString "(CCase" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . (showChar ' ' . (shows x3 . showChar
    ')'))))))
  showsPrec _ (C_CTyped x1 x2) = showString "(CTyped" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CRecConstr x1 x2) = showString "(CRecConstr" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CRecUpdate x1 x2) = showString "(CRecUpdate" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))

instance Read C_CExpr where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CVar x1, r1) | (_, r0) <-
    readQualified "AbstractCurry" "CVar" r, (x1, r1) <- readsPrec 11 r0]) s ++
    (readParen ((>) d 10) (\r -> [(C_CLit x1, r1) | (_, r0) <- readQualified
    "AbstractCurry" "CLit" r, (x1, r1) <- readsPrec 11 r0]) s ++ (readParen ((>)
    d 10) (\r -> [(C_CSymbol x1, r1) | (_, r0) <- readQualified "AbstractCurry"
    "CSymbol" r, (x1, r1) <- readsPrec 11 r0]) s ++ (readParen ((>) d 10) (\r ->
    [(C_CApply x1 x2, r2) | (_, r0) <- readQualified "AbstractCurry" "CApply" r,
    (x1, r1) <- readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1]) s ++ (readParen
    ((>) d 10) (\r -> [(C_CLambda x1 x2, r2) | (_, r0) <- readQualified
    "AbstractCurry" "CLambda" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <-
    readsPrec 11 r1]) s ++ (readParen ((>) d 10) (\r -> [(C_CLetDecl x1 x2
    , r2) | (_, r0) <- readQualified "AbstractCurry" "CLetDecl" r, (x1, r1) <-
    readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1]) s ++ (readParen ((>) d 10)
    (\r -> [(C_CDoExpr x1, r1) | (_, r0) <- readQualified "AbstractCurry"
    "CDoExpr" r, (x1, r1) <- readsPrec 11 r0]) s ++ (readParen ((>) d 10) (\r ->
    [(C_CListComp x1 x2, r2) | (_, r0) <- readQualified "AbstractCurry"
    "CListComp" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1])
    s ++ (readParen ((>) d 10) (\r -> [(C_CCase x1 x2 x3, r3) | (_, r0) <-
    readQualified "AbstractCurry" "CCase" r, (x1, r1) <- readsPrec 11 r0, (x2
    , r2) <- readsPrec 11 r1, (x3, r3) <- readsPrec 11 r2]) s ++ (readParen ((>)
    d 10) (\r -> [(C_CTyped x1 x2, r2) | (_, r0) <- readQualified
    "AbstractCurry" "CTyped" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <-
    readsPrec 11 r1]) s ++ (readParen ((>) d 10) (\r -> [(C_CRecConstr x1 x2
    , r2) | (_, r0) <- readQualified "AbstractCurry" "CRecConstr" r, (x1, r1) <-
    readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1]) s ++ readParen ((>) d 10)
    (\r -> [(C_CRecUpdate x1 x2, r2) | (_, r0) <- readQualified "AbstractCurry"
    "CRecUpdate" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <- readsPrec 11 r1])
    s))))))))))

instance NonDet C_CExpr where
  choiceCons = Choice_C_CExpr
  choicesCons = Choices_C_CExpr
  failCons = Fail_C_CExpr
  guardCons = Guard_C_CExpr
  try (Choice_C_CExpr cd i x y) = tryChoice cd i x y
  try (Choices_C_CExpr cd i xs) = tryChoices cd i xs
  try (Fail_C_CExpr cd info) = Fail cd info
  try (Guard_C_CExpr cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CExpr cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CExpr cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CExpr cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CExpr cd i _) = error
    ("AbstractCurry.CExpr.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CExpr cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CExpr cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CExpr where
  generate s c = Choices_C_CExpr c (freeID [1, 1, 1, 2, 2, 2, 1, 2, 3, 2, 2, 2]
    s) [C_CVar (generate (leftSupply s) c), C_CLit (generate (leftSupply s) c)
    , C_CSymbol (generate (leftSupply s) c), C_CApply (generate (leftSupply s)
    c) (generate (rightSupply s) c), C_CLambda (generate (leftSupply s) c)
    (generate (rightSupply s) c), C_CLetDecl (generate (leftSupply s) c)
    (generate (rightSupply s) c), C_CDoExpr (generate (leftSupply s) c)
    , C_CListComp (generate (leftSupply s) c) (generate (rightSupply s) c)
    , C_CCase (generate (leftSupply (leftSupply s)) c) (generate (rightSupply
    (leftSupply s)) c) (generate (rightSupply s) c), C_CTyped (generate
    (leftSupply s) c) (generate (rightSupply s) c), C_CRecConstr (generate
    (leftSupply s) c) (generate (rightSupply s) c), C_CRecUpdate (generate
    (leftSupply s) c) (generate (rightSupply s) c)]

instance NormalForm C_CExpr where
  ($!!) cont (C_CVar x1) d cs = ($!!) (\y1 d cs -> cont (C_CVar y1) d cs) x1 d
    cs
  ($!!) cont (C_CLit x1) d cs = ($!!) (\y1 d cs -> cont (C_CLit y1) d cs) x1 d
    cs
  ($!!) cont (C_CSymbol x1) d cs = ($!!) (\y1 d cs -> cont (C_CSymbol y1) d cs)
    x1 d cs
  ($!!) cont (C_CApply x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> cont
    (C_CApply y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CLambda x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> cont
    (C_CLambda y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CLetDecl x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CLetDecl y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CDoExpr x1) d cs = ($!!) (\y1 d cs -> cont (C_CDoExpr y1) d cs)
    x1 d cs
  ($!!) cont (C_CListComp x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CListComp y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CCase x1 x2 x3) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    ($!!) (\y3 d cs -> cont (C_CCase y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CTyped x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> cont
    (C_CTyped y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CRecConstr x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CRecConstr y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CRecUpdate x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_CRecUpdate y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_CExpr cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CExpr cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CExpr cd c e) d cs = guardCons cd c (($!!) cont e d (addCs
    c cs))
  ($!!) _ (Fail_C_CExpr cd info) _ _ = failCons cd info
  ($##) cont (C_CVar x1) d cs = ($##) (\y1 d cs -> cont (C_CVar y1) d cs) x1 d
    cs
  ($##) cont (C_CLit x1) d cs = ($##) (\y1 d cs -> cont (C_CLit y1) d cs) x1 d
    cs
  ($##) cont (C_CSymbol x1) d cs = ($##) (\y1 d cs -> cont (C_CSymbol y1) d cs)
    x1 d cs
  ($##) cont (C_CApply x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> cont
    (C_CApply y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CLambda x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> cont
    (C_CLambda y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CLetDecl x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CLetDecl y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CDoExpr x1) d cs = ($##) (\y1 d cs -> cont (C_CDoExpr y1) d cs)
    x1 d cs
  ($##) cont (C_CListComp x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CListComp y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CCase x1 x2 x3) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    ($##) (\y3 d cs -> cont (C_CCase y1 y2 y3) d cs) x3 d cs) x2 d cs) x1 d cs
  ($##) cont (C_CTyped x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> cont
    (C_CTyped y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CRecConstr x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CRecConstr y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CRecUpdate x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_CRecUpdate y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_CExpr cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CExpr cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CExpr cd c e) d cs = guardCons cd c (($##) cont e d (addCs
    c cs))
  ($##) _ (Fail_C_CExpr cd info) _ _ = failCons cd info
  showCons (C_CVar _) = "AbstractCurry.CVar _"
  showCons (C_CLit _) = "AbstractCurry.CLit _"
  showCons (C_CSymbol _) = "AbstractCurry.CSymbol _"
  showCons (C_CApply _ _) = "AbstractCurry.CApply _ _"
  showCons (C_CLambda _ _) = "AbstractCurry.CLambda _ _"
  showCons (C_CLetDecl _ _) = "AbstractCurry.CLetDecl _ _"
  showCons (C_CDoExpr _) = "AbstractCurry.CDoExpr _"
  showCons (C_CListComp _ _) = "AbstractCurry.CListComp _ _"
  showCons (C_CCase _ _ _) = "AbstractCurry.CCase _ _ _"
  showCons (C_CTyped _ _) = "AbstractCurry.CTyped _ _"
  showCons (C_CRecConstr _ _) = "AbstractCurry.CRecConstr _ _"
  showCons (C_CRecUpdate _ _) = "AbstractCurry.CRecUpdate _ _"
  showCons x = error ("AbstractCurry.CExpr.showCons: no constructor: " ++ show
    x)
  searchNF search cont (C_CVar x1) = search (\y1 -> cont (C_CVar y1)) x1
  searchNF search cont (C_CLit x1) = search (\y1 -> cont (C_CLit y1)) x1
  searchNF search cont (C_CSymbol x1) = search (\y1 -> cont (C_CSymbol y1)) x1
  searchNF search cont (C_CApply x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CApply y1 y2)) x2) x1
  searchNF search cont (C_CLambda x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CLambda y1 y2)) x2) x1
  searchNF search cont (C_CLetDecl x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CLetDecl y1 y2)) x2) x1
  searchNF search cont (C_CDoExpr x1) = search (\y1 -> cont (C_CDoExpr y1)) x1
  searchNF search cont (C_CListComp x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CListComp y1 y2)) x2) x1
  searchNF search cont (C_CCase x1 x2 x3) = search (\y1 -> search (\y2 -> search
    (\y3 -> cont (C_CCase y1 y2 y3)) x3) x2) x1
  searchNF search cont (C_CTyped x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CTyped y1 y2)) x2) x1
  searchNF search cont (C_CRecConstr x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CRecConstr y1 y2)) x2) x1
  searchNF search cont (C_CRecUpdate x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CRecUpdate y1 y2)) x2) x1
  searchNF _ _ x = error ("AbstractCurry.CExpr.searchNF: no constructor: " ++
    show x)

instance Unifiable C_CExpr where
  (=.=) (C_CVar x1) (C_CVar y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CLit x1) (C_CLit y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CSymbol x1) (C_CSymbol y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CApply x1 x2) (C_CApply y1 y2) d cs = (&) ((=:=) x1 y1 d cs) ((=:=)
    x2 y2 d cs) d cs
  (=.=) (C_CLambda x1 x2) (C_CLambda y1 y2) d cs = (&) ((=:=) x1 y1 d cs) ((=:=)
    x2 y2 d cs) d cs
  (=.=) (C_CLetDecl x1 x2) (C_CLetDecl y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) (C_CDoExpr x1) (C_CDoExpr y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CListComp x1 x2) (C_CListComp y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) (C_CCase x1 x2 x3) (C_CCase y1 y2 y3) d cs = (&) ((=:=) x1 y1 d cs) ((&)
    ((=:=) x2 y2 d cs) ((=:=) x3 y3 d cs) d cs) d cs
  (=.=) (C_CTyped x1 x2) (C_CTyped y1 y2) d cs = (&) ((=:=) x1 y1 d cs) ((=:=)
    x2 y2 d cs) d cs
  (=.=) (C_CRecConstr x1 x2) (C_CRecConstr y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) (C_CRecUpdate x1 x2) (C_CRecUpdate y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CVar x1) (C_CVar y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CLit x1) (C_CLit y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CSymbol x1) (C_CSymbol y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CApply x1 x2) (C_CApply y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CLambda x1 x2) (C_CLambda y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CLetDecl x1 x2) (C_CLetDecl y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CDoExpr x1) (C_CDoExpr y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CListComp x1 x2) (C_CListComp y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CCase x1 x2 x3) (C_CCase y1 y2 y3) d cs = (&) ((=:<=) x1 y1 d cs)
    ((&) ((=:<=) x2 y2 d cs) ((=:<=) x3 y3 d cs) d cs) d cs
  (=.<=) (C_CTyped x1 x2) (C_CTyped y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CRecConstr x1 x2) (C_CRecConstr y1 y2) d cs = (&) ((=:<=) x1 y1 d
    cs) ((=:<=) x2 y2 d cs) d cs
  (=.<=) (C_CRecUpdate x1 x2) (C_CRecUpdate y1 y2) d cs = (&) ((=:<=) x1 y1 d
    cs) ((=:<=) x2 y2 d cs) d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CVar x3) = (i :=: ChooseN 0 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CLit x3) = (i :=: ChooseN 1 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CSymbol x3) = (i :=: ChooseN 2 1) : concat [bind cd (leftID i)
    x3]
  bind cd i (C_CApply x3 x4) = (i :=: ChooseN 3 2) : concat [bind cd (leftID i)
    x3, bind cd (rightID i) x4]
  bind cd i (C_CLambda x3 x4) = (i :=: ChooseN 4 2) : concat [bind cd (leftID i)
    x3, bind cd (rightID i) x4]
  bind cd i (C_CLetDecl x3 x4) = (i :=: ChooseN 5 2) : concat [bind cd (leftID
    i) x3, bind cd (rightID i) x4]
  bind cd i (C_CDoExpr x3) = (i :=: ChooseN 6 1) : concat [bind cd (leftID i)
    x3]
  bind cd i (C_CListComp x3 x4) = (i :=: ChooseN 7 2) : concat [bind cd (leftID
    i) x3, bind cd (rightID i) x4]
  bind cd i (C_CCase x3 x4 x5) = (i :=: ChooseN 8 3) : concat [bind cd (leftID
    (leftID i)) x3, bind cd (rightID (leftID i)) x4, bind cd (rightID i) x5]
  bind cd i (C_CTyped x3 x4) = (i :=: ChooseN 9 2) : concat [bind cd (leftID i)
    x3, bind cd (rightID i) x4]
  bind cd i (C_CRecConstr x3 x4) = (i :=: ChooseN 10 2) : concat [bind cd
    (leftID i) x3, bind cd (rightID i) x4]
  bind cd i (C_CRecUpdate x3 x4) = (i :=: ChooseN 11 2) : concat [bind cd
    (leftID i) x3, bind cd (rightID i) x4]
  bind d i (Choice_C_CExpr cd j x y) = [ConstraintChoice cd j (bind d i x) (bind
    d i y)]
  bind d i (Choices_C_CExpr cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CExpr cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd j
    (map (bind d i) xs)]
  bind _ _ (Choices_C_CExpr cd i _) = error
    ("AbstractCurry.CExpr.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CExpr cd info) = [Unsolvable info]
  bind d i (Guard_C_CExpr cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CVar x3) = (i :=: ChooseN 0 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CLit x3) = (i :=: ChooseN 1 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CSymbol x3) = (i :=: ChooseN 2 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CApply x3 x4) = (i :=: ChooseN 3 2) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd (rightID i)
    x4)]
  lazyBind cd i (C_CLambda x3 x4) = (i :=: ChooseN 4 2) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd (rightID i)
    x4)]
  lazyBind cd i (C_CLetDecl x3 x4) = (i :=: ChooseN 5 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind cd i (C_CDoExpr x3) = (i :=: ChooseN 6 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CListComp x3 x4) = (i :=: ChooseN 7 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind cd i (C_CCase x3 x4 x5) = (i :=: ChooseN 8 3) : [leftID (leftID
    i) :=: LazyBind (lazyBind cd (leftID (leftID i)) x3), rightID (leftID i) :=:
    LazyBind (lazyBind cd (rightID (leftID i)) x4), rightID i :=: LazyBind
    (lazyBind cd (rightID i) x5)]
  lazyBind cd i (C_CTyped x3 x4) = (i :=: ChooseN 9 2) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd (rightID i)
    x4)]
  lazyBind cd i (C_CRecConstr x3 x4) = (i :=: ChooseN 10 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind cd i (C_CRecUpdate x3 x4) = (i :=: ChooseN 11 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind d i (Choice_C_CExpr cd j x y) = [ConstraintChoice cd j (lazyBind d i
    x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CExpr cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i cd
    j xs
  lazyBind d i (Choices_C_CExpr cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CExpr cd i _) = error
    ("AbstractCurry.CExpr.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CExpr cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CExpr cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CExpr where
  (=?=) (Choice_C_CExpr cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=) x z
    d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CExpr cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CExpr cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=) e y
    d (addCs c cs))
  (=?=) (Fail_C_CExpr cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CExpr cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=) z x
    d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CExpr cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CExpr cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=) y e
    d (addCs c cs))
  (=?=) _ (Fail_C_CExpr cd info) _ _ = failCons cd info
  (=?=) (C_CVar x1) (C_CVar y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CLit x1) (C_CLit y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CSymbol x1) (C_CSymbol y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CApply x1 x2) (C_CApply y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CLambda x1 x2) (C_CLambda y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CLetDecl x1 x2) (C_CLetDecl y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CDoExpr x1) (C_CDoExpr y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CListComp x1 x2) (C_CListComp y1 y2) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CCase x1 x2 x3) (C_CCase y1 y2 y3) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x2 y2 d cs) ((Curry_Prelude.=?=) x3 y3 d cs) d cs) d cs
  (=?=) (C_CTyped x1 x2) (C_CTyped y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CRecConstr x1 x2) (C_CRecConstr y1 y2) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CRecUpdate x1 x2) (C_CRecUpdate y1 y2) d cs =
    Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CExpr cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=) x z
    d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CExpr cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CExpr cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=) e y
    d (addCs c cs))
  (<?=) (Fail_C_CExpr cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CExpr cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=) z x
    d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CExpr cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CExpr cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=) y e
    d (addCs c cs))
  (<?=) _ (Fail_C_CExpr cd info) _ _ = failCons cd info
  (<?=) (C_CVar x1) (C_CVar y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CVar _) (C_CLit _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CSymbol _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CApply _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CLambda _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CLetDecl _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CDoExpr _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CListComp _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CCase _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CVar _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit x1) (C_CLit y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CLit _) (C_CSymbol _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CApply _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CLambda _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CLetDecl _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CDoExpr _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CListComp _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CCase _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLit _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol x1) (C_CSymbol y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CSymbol _) (C_CApply _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol _) (C_CLambda _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol _) (C_CLetDecl _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol _) (C_CDoExpr _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol _) (C_CListComp _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol _) (C_CCase _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSymbol _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CApply x1 x2) (C_CApply y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CApply _ _) (C_CLambda _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CApply _ _) (C_CLetDecl _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CApply _ _) (C_CDoExpr _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CApply _ _) (C_CListComp _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CApply _ _) (C_CCase _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CApply _ _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CApply _ _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CApply _ _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLambda x1 x2) (C_CLambda y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CLambda _ _) (C_CLetDecl _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLambda _ _) (C_CDoExpr _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLambda _ _) (C_CListComp _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLambda _ _) (C_CCase _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLambda _ _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLambda _ _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLambda _ _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLetDecl x1 x2) (C_CLetDecl y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CLetDecl _ _) (C_CDoExpr _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLetDecl _ _) (C_CListComp _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLetDecl _ _) (C_CCase _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLetDecl _ _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLetDecl _ _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CLetDecl _ _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CDoExpr x1) (C_CDoExpr y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CDoExpr _) (C_CListComp _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CDoExpr _) (C_CCase _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CDoExpr _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CDoExpr _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CDoExpr _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CListComp x1 x2) (C_CListComp y1 y2) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CListComp _ _) (C_CCase _ _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CListComp _ _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CListComp _ _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CListComp _ _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CCase x1 x2 x3) (C_CCase y1 y2 y3) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) (Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x2 y2 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x2 y2 d cs) ((Curry_Prelude.<?=) x3 y3 d cs) d cs) d
    cs) d cs) d cs
  (<?=) (C_CCase _ _ _) (C_CTyped _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CCase _ _ _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CCase _ _ _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CTyped x1 x2) (C_CTyped y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CTyped _ _) (C_CRecConstr _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CTyped _ _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CRecConstr x1 x2) (C_CRecConstr y1 y2) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CRecConstr _ _) (C_CRecUpdate _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CRecUpdate x1 x2) (C_CRecUpdate y1 y2) d cs =
    Curry_Prelude.d_OP_bar_bar (Curry_Prelude.d_OP_lt x1 y1 d cs)
    (Curry_Prelude.d_OP_amp_amp ((Curry_Prelude.=?=) x1 y1 d cs)
    ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CLiteral
  = C_CIntc Curry_Prelude.C_Int
  | C_CFloatc Curry_Prelude.C_Float
  | C_CCharc Curry_Prelude.C_Char
  | C_CStringc (Curry_Prelude.OP_List Curry_Prelude.C_Char)
  | Choice_C_CLiteral Cover ID C_CLiteral C_CLiteral
  | Choices_C_CLiteral Cover ID [C_CLiteral]
  | Fail_C_CLiteral Cover FailInfo
  | Guard_C_CLiteral Cover Constraints C_CLiteral

instance Show C_CLiteral where
  showsPrec d (Choice_C_CLiteral cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CLiteral cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CLiteral cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CLiteral cd info) = showChar '!'
  showsPrec _ (C_CIntc x1) = showString "(CIntc" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CFloatc x1) = showString "(CFloatc" . (showChar ' ' . (shows
    x1 . showChar ')'))
  showsPrec _ (C_CCharc x1) = showString "(CCharc" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CStringc x1) = showString "(CStringc" . (showChar ' ' . (shows
    x1 . showChar ')'))

instance Read C_CLiteral where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CIntc x1, r1) | (_, r0) <-
    readQualified "AbstractCurry" "CIntc" r, (x1, r1) <- readsPrec 11 r0]) s ++
    (readParen ((>) d 10) (\r -> [(C_CFloatc x1, r1) | (_, r0) <- readQualified
    "AbstractCurry" "CFloatc" r, (x1, r1) <- readsPrec 11 r0]) s ++ (readParen
    ((>) d 10) (\r -> [(C_CCharc x1, r1) | (_, r0) <- readQualified
    "AbstractCurry" "CCharc" r, (x1, r1) <- readsPrec 11 r0]) s ++ readParen
    ((>) d 10) (\r -> [(C_CStringc x1, r1) | (_, r0) <- readQualified
    "AbstractCurry" "CStringc" r, (x1, r1) <- readsPrec 11 r0]) s))

instance NonDet C_CLiteral where
  choiceCons = Choice_C_CLiteral
  choicesCons = Choices_C_CLiteral
  failCons = Fail_C_CLiteral
  guardCons = Guard_C_CLiteral
  try (Choice_C_CLiteral cd i x y) = tryChoice cd i x y
  try (Choices_C_CLiteral cd i xs) = tryChoices cd i xs
  try (Fail_C_CLiteral cd info) = Fail cd info
  try (Guard_C_CLiteral cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CLiteral cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CLiteral cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CLiteral cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CLiteral cd i _) = error
    ("AbstractCurry.CLiteral.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CLiteral cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CLiteral cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CLiteral where
  generate s c = Choices_C_CLiteral c (freeID [1, 1, 1, 1] s) [C_CIntc (generate
    (leftSupply s) c), C_CFloatc (generate (leftSupply s) c), C_CCharc (generate
    (leftSupply s) c), C_CStringc (generate (leftSupply s) c)]

instance NormalForm C_CLiteral where
  ($!!) cont (C_CIntc x1) d cs = ($!!) (\y1 d cs -> cont (C_CIntc y1) d cs) x1 d
    cs
  ($!!) cont (C_CFloatc x1) d cs = ($!!) (\y1 d cs -> cont (C_CFloatc y1) d cs)
    x1 d cs
  ($!!) cont (C_CCharc x1) d cs = ($!!) (\y1 d cs -> cont (C_CCharc y1) d cs) x1
    d cs
  ($!!) cont (C_CStringc x1) d cs = ($!!) (\y1 d cs -> cont (C_CStringc y1) d
    cs) x1 d cs
  ($!!) cont (Choice_C_CLiteral cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CLiteral cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CLiteral cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CLiteral cd info) _ _ = failCons cd info
  ($##) cont (C_CIntc x1) d cs = ($##) (\y1 d cs -> cont (C_CIntc y1) d cs) x1 d
    cs
  ($##) cont (C_CFloatc x1) d cs = ($##) (\y1 d cs -> cont (C_CFloatc y1) d cs)
    x1 d cs
  ($##) cont (C_CCharc x1) d cs = ($##) (\y1 d cs -> cont (C_CCharc y1) d cs) x1
    d cs
  ($##) cont (C_CStringc x1) d cs = ($##) (\y1 d cs -> cont (C_CStringc y1) d
    cs) x1 d cs
  ($##) cont (Choice_C_CLiteral cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CLiteral cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CLiteral cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CLiteral cd info) _ _ = failCons cd info
  showCons (C_CIntc _) = "AbstractCurry.CIntc _"
  showCons (C_CFloatc _) = "AbstractCurry.CFloatc _"
  showCons (C_CCharc _) = "AbstractCurry.CCharc _"
  showCons (C_CStringc _) = "AbstractCurry.CStringc _"
  showCons x = error ("AbstractCurry.CLiteral.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CIntc x1) = search (\y1 -> cont (C_CIntc y1)) x1
  searchNF search cont (C_CFloatc x1) = search (\y1 -> cont (C_CFloatc y1)) x1
  searchNF search cont (C_CCharc x1) = search (\y1 -> cont (C_CCharc y1)) x1
  searchNF search cont (C_CStringc x1) = search (\y1 -> cont (C_CStringc y1)) x1
  searchNF _ _ x = error ("AbstractCurry.CLiteral.searchNF: no constructor: " ++
    show x)

instance Unifiable C_CLiteral where
  (=.=) (C_CIntc x1) (C_CIntc y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CFloatc x1) (C_CFloatc y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CCharc x1) (C_CCharc y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CStringc x1) (C_CStringc y1) d cs = (=:=) x1 y1 d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CIntc x1) (C_CIntc y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CFloatc x1) (C_CFloatc y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CCharc x1) (C_CCharc y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CStringc x1) (C_CStringc y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CIntc x3) = (i :=: ChooseN 0 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CFloatc x3) = (i :=: ChooseN 1 1) : concat [bind cd (leftID i)
    x3]
  bind cd i (C_CCharc x3) = (i :=: ChooseN 2 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CStringc x3) = (i :=: ChooseN 3 1) : concat [bind cd (leftID i)
    x3]
  bind d i (Choice_C_CLiteral cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CLiteral cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CLiteral cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd
    j (map (bind d i) xs)]
  bind _ _ (Choices_C_CLiteral cd i _) = error
    ("AbstractCurry.CLiteral.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CLiteral cd info) = [Unsolvable info]
  bind d i (Guard_C_CLiteral cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CIntc x3) = (i :=: ChooseN 0 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CFloatc x3) = (i :=: ChooseN 1 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CCharc x3) = (i :=: ChooseN 2 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CStringc x3) = (i :=: ChooseN 3 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind d i (Choice_C_CLiteral cd j x y) = [ConstraintChoice cd j (lazyBind d
    i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CLiteral cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CLiteral cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CLiteral cd i _) = error
    ("AbstractCurry.CLiteral.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CLiteral cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CLiteral cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CLiteral where
  (=?=) (Choice_C_CLiteral cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=) x
    z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CLiteral cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CLiteral cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=) e
    y d (addCs c cs))
  (=?=) (Fail_C_CLiteral cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CLiteral cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=) z
    x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CLiteral cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CLiteral cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=) y
    e d (addCs c cs))
  (=?=) _ (Fail_C_CLiteral cd info) _ _ = failCons cd info
  (=?=) (C_CIntc x1) (C_CIntc y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CFloatc x1) (C_CFloatc y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CCharc x1) (C_CCharc y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CStringc x1) (C_CStringc y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CLiteral cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=) x
    z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CLiteral cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CLiteral cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=) e
    y d (addCs c cs))
  (<?=) (Fail_C_CLiteral cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CLiteral cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=) z
    x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CLiteral cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CLiteral cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=) y
    e d (addCs c cs))
  (<?=) _ (Fail_C_CLiteral cd info) _ _ = failCons cd info
  (<?=) (C_CIntc x1) (C_CIntc y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CIntc _) (C_CFloatc _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CIntc _) (C_CCharc _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CIntc _) (C_CStringc _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CFloatc x1) (C_CFloatc y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CFloatc _) (C_CCharc _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CFloatc _) (C_CStringc _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CCharc x1) (C_CCharc y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CCharc _) (C_CStringc _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CStringc x1) (C_CStringc y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CStatement
  = C_CSExpr C_CExpr
  | C_CSPat C_CPattern C_CExpr
  | C_CSLet (Curry_Prelude.OP_List C_CLocalDecl)
  | Choice_C_CStatement Cover ID C_CStatement C_CStatement
  | Choices_C_CStatement Cover ID [C_CStatement]
  | Fail_C_CStatement Cover FailInfo
  | Guard_C_CStatement Cover Constraints C_CStatement

instance Show C_CStatement where
  showsPrec d (Choice_C_CStatement cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CStatement cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CStatement cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CStatement cd info) = showChar '!'
  showsPrec _ (C_CSExpr x1) = showString "(CSExpr" . (showChar ' ' . (shows x1 .
    showChar ')'))
  showsPrec _ (C_CSPat x1 x2) = showString "(CSPat" . (showChar ' ' . (shows
    x1 . (showChar ' ' . (shows x2 . showChar ')'))))
  showsPrec _ (C_CSLet x1) = showString "(CSLet" . (showChar ' ' . (shows x1 .
    showChar ')'))

instance Read C_CStatement where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_CSExpr x1, r1) | (_, r0) <-
    readQualified "AbstractCurry" "CSExpr" r, (x1, r1) <- readsPrec 11 r0]) s ++
    (readParen ((>) d 10) (\r -> [(C_CSPat x1 x2, r2) | (_, r0) <- readQualified
    "AbstractCurry" "CSPat" r, (x1, r1) <- readsPrec 11 r0, (x2, r2) <-
    readsPrec 11 r1]) s ++ readParen ((>) d 10) (\r -> [(C_CSLet x1, r1) | (_
    , r0) <- readQualified "AbstractCurry" "CSLet" r, (x1, r1) <- readsPrec 11
    r0]) s)

instance NonDet C_CStatement where
  choiceCons = Choice_C_CStatement
  choicesCons = Choices_C_CStatement
  failCons = Fail_C_CStatement
  guardCons = Guard_C_CStatement
  try (Choice_C_CStatement cd i x y) = tryChoice cd i x y
  try (Choices_C_CStatement cd i xs) = tryChoices cd i xs
  try (Fail_C_CStatement cd info) = Fail cd info
  try (Guard_C_CStatement cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CStatement cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CStatement cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CStatement cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CStatement cd i _) = error
    ("AbstractCurry.CStatement.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CStatement cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CStatement cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CStatement where
  generate s c = Choices_C_CStatement c (freeID [1, 2, 1] s) [C_CSExpr (generate
    (leftSupply s) c), C_CSPat (generate (leftSupply s) c) (generate
    (rightSupply s) c), C_CSLet (generate (leftSupply s) c)]

instance NormalForm C_CStatement where
  ($!!) cont (C_CSExpr x1) d cs = ($!!) (\y1 d cs -> cont (C_CSExpr y1) d cs) x1
    d cs
  ($!!) cont (C_CSPat x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs -> cont
    (C_CSPat y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (C_CSLet x1) d cs = ($!!) (\y1 d cs -> cont (C_CSLet y1) d cs) x1 d
    cs
  ($!!) cont (Choice_C_CStatement cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CStatement cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CStatement cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CStatement cd info) _ _ = failCons cd info
  ($##) cont (C_CSExpr x1) d cs = ($##) (\y1 d cs -> cont (C_CSExpr y1) d cs) x1
    d cs
  ($##) cont (C_CSPat x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs -> cont
    (C_CSPat y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (C_CSLet x1) d cs = ($##) (\y1 d cs -> cont (C_CSLet y1) d cs) x1 d
    cs
  ($##) cont (Choice_C_CStatement cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CStatement cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CStatement cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CStatement cd info) _ _ = failCons cd info
  showCons (C_CSExpr _) = "AbstractCurry.CSExpr _"
  showCons (C_CSPat _ _) = "AbstractCurry.CSPat _ _"
  showCons (C_CSLet _) = "AbstractCurry.CSLet _"
  showCons x = error ("AbstractCurry.CStatement.showCons: no constructor: " ++
    show x)
  searchNF search cont (C_CSExpr x1) = search (\y1 -> cont (C_CSExpr y1)) x1
  searchNF search cont (C_CSPat x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_CSPat y1 y2)) x2) x1
  searchNF search cont (C_CSLet x1) = search (\y1 -> cont (C_CSLet y1)) x1
  searchNF _ _ x = error
    ("AbstractCurry.CStatement.searchNF: no constructor: " ++ show x)

instance Unifiable C_CStatement where
  (=.=) (C_CSExpr x1) (C_CSExpr y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_CSPat x1 x2) (C_CSPat y1 y2) d cs = (&) ((=:=) x1 y1 d cs) ((=:=) x2
    y2 d cs) d cs
  (=.=) (C_CSLet x1) (C_CSLet y1) d cs = (=:=) x1 y1 d cs
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_CSExpr x1) (C_CSExpr y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_CSPat x1 x2) (C_CSPat y1 y2) d cs = (&) ((=:<=) x1 y1 d cs) ((=:<=)
    x2 y2 d cs) d cs
  (=.<=) (C_CSLet x1) (C_CSLet y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i (C_CSExpr x3) = (i :=: ChooseN 0 1) : concat [bind cd (leftID i) x3]
  bind cd i (C_CSPat x3 x4) = (i :=: ChooseN 1 2) : concat [bind cd (leftID i)
    x3, bind cd (rightID i) x4]
  bind cd i (C_CSLet x3) = (i :=: ChooseN 2 1) : concat [bind cd (leftID i) x3]
  bind d i (Choice_C_CStatement cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CStatement cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j
    xs
  bind d i (Choices_C_CStatement cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CStatement cd i _) = error
    ("AbstractCurry.CStatement.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CStatement cd info) = [Unsolvable info]
  bind d i (Guard_C_CStatement cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_CSExpr x3) = (i :=: ChooseN 0 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_CSPat x3 x4) = (i :=: ChooseN 1 2) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd (rightID i)
    x4)]
  lazyBind cd i (C_CSLet x3) = (i :=: ChooseN 2 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind d i (Choice_C_CStatement cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CStatement cd j@(FreeID _ _) xs) = lazyBindOrNarrow d
    i cd j xs
  lazyBind d i (Choices_C_CStatement cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CStatement cd i _) = error
    ("AbstractCurry.CStatement.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CStatement cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CStatement cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CStatement where
  (=?=) (Choice_C_CStatement cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CStatement cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CStatement cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CStatement cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CStatement cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CStatement cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CStatement cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CStatement cd info) _ _ = failCons cd info
  (=?=) (C_CSExpr x1) (C_CSExpr y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) (C_CSPat x1 x2) (C_CSPat y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.=?=) x2 y2 d cs) d cs
  (=?=) (C_CSLet x1) (C_CSLet y1) d cs = (Curry_Prelude.=?=) x1 y1 d cs
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CStatement cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CStatement cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CStatement cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CStatement cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CStatement cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CStatement cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CStatement cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CStatement cd info) _ _ = failCons cd info
  (<?=) (C_CSExpr x1) (C_CSExpr y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) (C_CSExpr _) (C_CSPat _ _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSExpr _) (C_CSLet _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSPat x1 x2) (C_CSPat y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp
    ((Curry_Prelude.=?=) x1 y1 d cs) ((Curry_Prelude.<?=) x2 y2 d cs) d cs) d cs
  (<?=) (C_CSPat _ _) (C_CSLet _) _ _ = Curry_Prelude.C_True
  (<?=) (C_CSLet x1) (C_CSLet y1) d cs = (Curry_Prelude.<?=) x1 y1 d cs
  (<?=) _ _ d _ = Curry_Prelude.C_False

data C_CCaseType
  = C_CRigid
  | C_CFlex
  | Choice_C_CCaseType Cover ID C_CCaseType C_CCaseType
  | Choices_C_CCaseType Cover ID [C_CCaseType]
  | Fail_C_CCaseType Cover FailInfo
  | Guard_C_CCaseType Cover Constraints C_CCaseType

instance Show C_CCaseType where
  showsPrec d (Choice_C_CCaseType cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_CCaseType cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_CCaseType cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_CCaseType cd info) = showChar '!'
  showsPrec _ C_CRigid = showString "CRigid"
  showsPrec _ C_CFlex = showString "CFlex"

instance Read C_CCaseType where
  readsPrec _ s = readParen False (\r -> [(C_CRigid, r0) | (_, r0) <-
    readQualified "AbstractCurry" "CRigid" r]) s ++ readParen False (\r ->
    [(C_CFlex, r0) | (_, r0) <- readQualified "AbstractCurry" "CFlex" r]) s

instance NonDet C_CCaseType where
  choiceCons = Choice_C_CCaseType
  choicesCons = Choices_C_CCaseType
  failCons = Fail_C_CCaseType
  guardCons = Guard_C_CCaseType
  try (Choice_C_CCaseType cd i x y) = tryChoice cd i x y
  try (Choices_C_CCaseType cd i xs) = tryChoices cd i xs
  try (Fail_C_CCaseType cd info) = Fail cd info
  try (Guard_C_CCaseType cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_CCaseType cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_CCaseType cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_CCaseType cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_CCaseType cd i _) = error
    ("AbstractCurry.CCaseType.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_CCaseType cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_CCaseType cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance Generable C_CCaseType where
  generate s c = Choices_C_CCaseType c (freeID [0, 0] s) [C_CRigid, C_CFlex]

instance NormalForm C_CCaseType where
  ($!!) cont C_CRigid d cs = cont C_CRigid d cs
  ($!!) cont C_CFlex d cs = cont C_CFlex d cs
  ($!!) cont (Choice_C_CCaseType cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_CCaseType cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_CCaseType cd c e) d cs = guardCons cd c (($!!) cont e d
    (addCs c cs))
  ($!!) _ (Fail_C_CCaseType cd info) _ _ = failCons cd info
  ($##) cont C_CRigid d cs = cont C_CRigid d cs
  ($##) cont C_CFlex d cs = cont C_CFlex d cs
  ($##) cont (Choice_C_CCaseType cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_CCaseType cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_CCaseType cd c e) d cs = guardCons cd c (($##) cont e d
    (addCs c cs))
  ($##) _ (Fail_C_CCaseType cd info) _ _ = failCons cd info
  showCons C_CRigid = "AbstractCurry.CRigid"
  showCons C_CFlex = "AbstractCurry.CFlex"
  showCons x = error ("AbstractCurry.CCaseType.showCons: no constructor: " ++
    show x)
  searchNF _ cont C_CRigid = cont C_CRigid
  searchNF _ cont C_CFlex = cont C_CFlex
  searchNF _ _ x = error
    ("AbstractCurry.CCaseType.searchNF: no constructor: " ++ show x)

instance Unifiable C_CCaseType where
  (=.=) C_CRigid C_CRigid d cs = C_Success
  (=.=) C_CFlex C_CFlex d cs = C_Success
  (=.=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons b))
  (=.<=) C_CRigid C_CRigid d cs = C_Success
  (=.<=) C_CFlex C_CFlex d cs = C_Success
  (=.<=) a b cd _ = Fail_C_Success cd (unificationFail (showCons a) (showCons
    b))
  bind cd i C_CRigid = (i :=: ChooseN 0 0) : concat []
  bind cd i C_CFlex = (i :=: ChooseN 1 0) : concat []
  bind d i (Choice_C_CCaseType cd j x y) = [ConstraintChoice cd j (bind d i x)
    (bind d i y)]
  bind d i (Choices_C_CCaseType cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_CCaseType cd j@(NarrowedID _ _) xs) = [ConstraintChoices
    cd j (map (bind d i) xs)]
  bind _ _ (Choices_C_CCaseType cd i _) = error
    ("AbstractCurry.CCaseType.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_CCaseType cd info) = [Unsolvable info]
  bind d i (Guard_C_CCaseType cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i C_CRigid = (i :=: ChooseN 0 0) : []
  lazyBind cd i C_CFlex = (i :=: ChooseN 1 0) : []
  lazyBind d i (Choice_C_CCaseType cd j x y) = [ConstraintChoice cd j (lazyBind
    d i x) (lazyBind d i y)]
  lazyBind d i (Choices_C_CCaseType cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i
    cd j xs
  lazyBind d i (Choices_C_CCaseType cd j@(NarrowedID _ _) xs) =
    [ConstraintChoices cd j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_CCaseType cd i _) = error
    ("AbstractCurry.CCaseType.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_CCaseType cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_CCaseType cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance Curry_Prelude.Curry C_CCaseType where
  (=?=) (Choice_C_CCaseType cd i x y) z d cs = narrow cd i ((Curry_Prelude.=?=)
    x z d cs) ((Curry_Prelude.=?=) y z d cs)
  (=?=) (Choices_C_CCaseType cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) x y d cs) xs
  (=?=) (Guard_C_CCaseType cd c e) y d cs = guardCons cd c ((Curry_Prelude.=?=)
    e y d (addCs c cs))
  (=?=) (Fail_C_CCaseType cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_CCaseType cd i x y) d cs = narrow cd i ((Curry_Prelude.=?=)
    z x d cs) ((Curry_Prelude.=?=) z y d cs)
  (=?=) y (Choices_C_CCaseType cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.=?=) y x d cs) xs
  (=?=) y (Guard_C_CCaseType cd c e) d cs = guardCons cd c ((Curry_Prelude.=?=)
    y e d (addCs c cs))
  (=?=) _ (Fail_C_CCaseType cd info) _ _ = failCons cd info
  (=?=) C_CRigid C_CRigid d cs = Curry_Prelude.C_True
  (=?=) C_CFlex C_CFlex d cs = Curry_Prelude.C_True
  (=?=) _ _ d _ = Curry_Prelude.C_False
  (<?=) (Choice_C_CCaseType cd i x y) z d cs = narrow cd i ((Curry_Prelude.<?=)
    x z d cs) ((Curry_Prelude.<?=) y z d cs)
  (<?=) (Choices_C_CCaseType cd i xs) y d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) x y d cs) xs
  (<?=) (Guard_C_CCaseType cd c e) y d cs = guardCons cd c ((Curry_Prelude.<?=)
    e y d (addCs c cs))
  (<?=) (Fail_C_CCaseType cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_CCaseType cd i x y) d cs = narrow cd i ((Curry_Prelude.<?=)
    z x d cs) ((Curry_Prelude.<?=) z y d cs)
  (<?=) y (Choices_C_CCaseType cd i xs) d cs = narrows cs cd i (\x ->
    (Curry_Prelude.<?=) y x d cs) xs
  (<?=) y (Guard_C_CCaseType cd c e) d cs = guardCons cd c ((Curry_Prelude.<?=)
    y e d (addCs c cs))
  (<?=) _ (Fail_C_CCaseType cd info) _ _ = failCons cd info
  (<?=) C_CRigid C_CRigid d cs = Curry_Prelude.C_True
  (<?=) C_CRigid C_CFlex _ _ = Curry_Prelude.C_True
  (<?=) C_CFlex C_CFlex d cs = Curry_Prelude.C_True
  (<?=) _ _ d _ = Curry_Prelude.C_False

d_C_version :: Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_version cd cs = Curry_Prelude.toCurryString "AbstractCurry 1.0"

d_C_readCurry :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover
  -> ConstStore -> Curry_Prelude.C_IO C_CurryProg
d_C_readCurry x1 cd cs = d_C_readCurryWithParseOptions x1
  (Curry_Distribution.d_C_setQuiet Curry_Prelude.C_True
  (Curry_Distribution.d_C_defaultParams cd cs) cd cs) cd cs

d_C_readUntypedCurry :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover
  -> ConstStore -> Curry_Prelude.C_IO C_CurryProg
d_C_readUntypedCurry x1 cd cs = d_C_readUntypedCurryWithParseOptions x1
  (Curry_Distribution.d_C_setQuiet Curry_Prelude.C_True
  (Curry_Distribution.d_C_defaultParams cd cs) cd cs) cd cs

d_C_readCurryWithParseOptions :: Curry_Prelude.OP_List Curry_Prelude.C_Char
  -> Curry_Distribution.C_FrontendParams -> Cover -> ConstStore
  -> Curry_Prelude.C_IO C_CurryProg
d_C_readCurryWithParseOptions x1 x2 cd cs = Curry_Prelude.d_OP_gt_gt_eq
  (Curry_Prelude.d_OP_gt_gt_eq
  (Curry_Distribution.d_C_lookupModuleSourceInLoadPath x1 cd cs)
  (Curry_Prelude.d_OP_dot Curry_Prelude.d_C_return (Curry_Prelude.d_C_maybe
  Curry_Prelude.C_Nothing (Curry_Prelude.d_OP_dot (acceptCs id
  Curry_Prelude.C_Just) Curry_Prelude.d_C_fst cd cs)) cd cs) cd cs)
  (d_OP_readCurryWithParseOptions_dot_uscore_hash_lambda1 x2 x1) cd cs

d_OP_readCurryWithParseOptions_dot_uscore_hash_lambda1 :: Curry_Distribution.C_FrontendParams
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Maybe
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore
  -> Curry_Prelude.C_IO C_CurryProg
d_OP_readCurryWithParseOptions_dot_uscore_hash_lambda1 x1 x2 x3 cd cs =
  Curry_Prelude.d_OP_gt_gt (Curry_Prelude.d_OP_dollar (Curry_Prelude.d_C_unless
  (Curry_Maybe.d_C_isNothing x3 cd cs))
  (Curry_Distribution.d_C_callFrontendWithParams Curry_Distribution.C_ACY x1 x2
  cd cs) cd cs) (Curry_Prelude.d_OP_gt_gt_eq
  (Curry_Distribution.d_C_findFileInLoadPath (d_C_abstractCurryFileName x2 cd
  cs) cd cs)
  d_OP_readCurryWithParseOptions_dot_uscore_hash_lambda1_dot_uscore_hash_lambda2
  cd cs) cd cs

d_OP_readCurryWithParseOptions_dot_uscore_hash_lambda1_dot_uscore_hash_lambda2 :: Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_IO C_CurryProg
d_OP_readCurryWithParseOptions_dot_uscore_hash_lambda1_dot_uscore_hash_lambda2 x1 cd cs =
  d_C_readAbstractCurryFile x1 cd cs

d_C_readUntypedCurryWithParseOptions :: Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Distribution.C_FrontendParams -> Cover
  -> ConstStore -> Curry_Prelude.C_IO C_CurryProg
d_C_readUntypedCurryWithParseOptions x1 x2 cd cs = Curry_Prelude.d_OP_gt_gt_eq
  (Curry_Prelude.d_OP_gt_gt_eq
  (Curry_Distribution.d_C_lookupModuleSourceInLoadPath x1 cd cs)
  (Curry_Prelude.d_OP_dot Curry_Prelude.d_C_return (Curry_Prelude.d_C_maybe
  Curry_Prelude.C_Nothing (Curry_Prelude.d_OP_dot (acceptCs id
  Curry_Prelude.C_Just) Curry_Prelude.d_C_fst cd cs)) cd cs) cd cs)
  (d_OP_readUntypedCurryWithParseOptions_dot_uscore_hash_lambda3 x2 x1) cd cs

d_OP_readUntypedCurryWithParseOptions_dot_uscore_hash_lambda3 :: Curry_Distribution.C_FrontendParams
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Maybe
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore
  -> Curry_Prelude.C_IO C_CurryProg
d_OP_readUntypedCurryWithParseOptions_dot_uscore_hash_lambda3 x1 x2 x3 cd cs =
  Curry_Prelude.d_OP_gt_gt (Curry_Prelude.d_OP_dollar (Curry_Prelude.d_C_unless
  (Curry_Maybe.d_C_isNothing x3 cd cs))
  (Curry_Distribution.d_C_callFrontendWithParams Curry_Distribution.C_UACY x1 x2
  cd cs) cd cs) (Curry_Prelude.d_OP_gt_gt_eq
  (Curry_Distribution.d_C_findFileInLoadPath (d_C_untypedAbstractCurryFileName
  x2 cd cs) cd cs)
  d_OP_readUntypedCurryWithParseOptions_dot_uscore_hash_lambda3_dot_uscore_hash_lambda4
  cd cs) cd cs

d_OP_readUntypedCurryWithParseOptions_dot_uscore_hash_lambda3_dot_uscore_hash_lambda4 :: Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_IO C_CurryProg
d_OP_readUntypedCurryWithParseOptions_dot_uscore_hash_lambda3_dot_uscore_hash_lambda4 x1 cd cs =
  d_C_readAbstractCurryFile x1 cd cs

d_C_abstractCurryFileName :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover
  -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_abstractCurryFileName x1 cd cs = Curry_Prelude.d_C_apply
  (Curry_Prelude.d_C_apply (Curry_FilePath.d_OP_lt_dot_gt cd cs)
  (Curry_Distribution.d_C_inCurrySubdir (Curry_Distribution.d_C_stripCurrySuffix
  x1 cd cs) cd cs) cd cs) (Curry_Prelude.toCurryString "acy") cd cs

d_C_untypedAbstractCurryFileName :: Curry_Prelude.OP_List Curry_Prelude.C_Char
  -> Cover -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_untypedAbstractCurryFileName x1 cd cs = Curry_Prelude.d_C_apply
  (Curry_Prelude.d_C_apply (Curry_FilePath.d_OP_lt_dot_gt cd cs)
  (Curry_Distribution.d_C_inCurrySubdir (Curry_Distribution.d_C_stripCurrySuffix
  x1 cd cs) cd cs) cd cs) (Curry_Prelude.toCurryString "uacy") cd cs

d_C_readAbstractCurryFile :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover
  -> ConstStore -> Curry_Prelude.C_IO C_CurryProg
d_C_readAbstractCurryFile x1 cd cs = Curry_Prelude.d_OP_gt_gt_eq
  (Curry_Directory.d_C_doesFileExist x1 cd cs)
  (d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6 x1) cd cs

d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20 ::(Curry_Prelude.Curry
  t0) => Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover -> ConstStore
  -> Curry_Prelude.C_IO t0
d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20 x1 cd cs =
  Curry_Prelude.d_OP_gt_gt_eq (Curry_Prelude.d_C_readFile x1 cd cs)
  (d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5
  x1) cd cs

d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5 ::(Curry_Prelude.Curry
  t0) => Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.C_IO t0
d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5 x1 x2 cd cs =
  let x3 = Curry_Prelude.d_C_apply (Curry_Prelude.d_C_break
        (Curry_Prelude.d_C_flip (acceptCs id Curry_Prelude.d_OP_eq_eq)
        (Curry_Prelude.C_Char '\n'#)) cd cs) x2 cd cs
      x4 = d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP2_hash_line1
        x3 cd cs
      x5 = d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP3_hash_lines
        x3 cd cs
  in d_C__case_5 x4 x1 x5 (Curry_Prelude.d_OP_eq_eq x4
  (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "{- ")
  (Curry_Prelude.d_OP_plus_plus (d_C_version cd cs) (Curry_Prelude.toCurryString
  " -}") cd cs) cd cs) cd cs) cd cs

d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP2_hash_line1 :: Curry_Prelude.OP_Tuple2
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List
  Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP2_hash_line1 x1 cd cs =
  case x1 of
  Curry_Prelude.OP_Tuple2 x2 x3 -> x2
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i
    (d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP2_hash_line1
    l cd cs)
    (d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP2_hash_line1
    r cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z ->
    d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP2_hash_line1
    z cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c
    (d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP2_hash_line1
    e cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "AbstractCurry.readAbstractCurryFile.readExistingACY.20._#lambda5._#selFP2#line1"
    [show x1] info)
  _ -> failCons cd (consFail
    "AbstractCurry.readAbstractCurryFile.readExistingACY.20._#lambda5._#selFP2#line1"
    (showCons x1))

d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP3_hash_lines :: Curry_Prelude.OP_Tuple2
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List
  Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP3_hash_lines x1 cd cs =
  case x1 of
  Curry_Prelude.OP_Tuple2 x2 x3 -> x3
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i
    (d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP3_hash_lines
    l cd cs)
    (d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP3_hash_lines
    r cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z ->
    d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP3_hash_lines
    z cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c
    (d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20_dot_uscore_hash_lambda5_dot_uscore_hash_selFP3_hash_lines
    e cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "AbstractCurry.readAbstractCurryFile.readExistingACY.20._#lambda5._#selFP3#lines"
    [show x1] info)
  _ -> failCons cd (consFail
    "AbstractCurry.readAbstractCurryFile.readExistingACY.20._#lambda5._#selFP3#lines"
    (showCons x1))

d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6 ::(Curry_Prelude.Curry
  t0) => Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool
  -> Cover -> ConstStore -> Curry_Prelude.C_IO t0
d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6 x1 x2 cd cs = case x2 of
  Curry_Prelude.C_True -> d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20
    x1 cd cs
  Curry_Prelude.C_False -> let x3 = Curry_Distribution.d_C_inCurrySubdir x1 cd
                                 cs
    in Curry_Prelude.d_OP_gt_gt_eq (Curry_Directory.d_C_doesFileExist x3 cd cs)
    (d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6_dot_uscore_hash_lambda7
    x1 x3) cd cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i
    (d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6 x1 l cd cs)
    (d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6 x1 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z ->
    d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6 x1 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c
    (d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6 x1 e cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry.readAbstractCurryFile._#lambda6" [show x1, show x2] info)
  _ -> failCons cd (consFail "AbstractCurry.readAbstractCurryFile._#lambda6"
    (showCons x2))

d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6_dot_uscore_hash_lambda7 ::(Curry_Prelude.Curry
  t0) => Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore
  -> Curry_Prelude.C_IO t0
d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6_dot_uscore_hash_lambda7 x1 x2 x3 cd cs =
  case x3 of
  Curry_Prelude.C_True -> d_OP_readAbstractCurryFile_dot_readExistingACY_dot_20
    x2 cd cs
  Curry_Prelude.C_False -> Curry_Prelude.d_C_error (Curry_Prelude.d_OP_plus_plus
    (Curry_Prelude.toCurryString "EXISTENCE ERROR: AbstractCurry file '")
    (Curry_Prelude.d_OP_plus_plus x1 (Curry_Prelude.toCurryString
    "' does not exist") cd cs) cd cs) cd cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i
    (d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6_dot_uscore_hash_lambda7
    x1 x2 l cd cs)
    (d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6_dot_uscore_hash_lambda7
    x1 x2 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z ->
    d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6_dot_uscore_hash_lambda7
    x1 x2 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c
    (d_OP_readAbstractCurryFile_dot_uscore_hash_lambda6_dot_uscore_hash_lambda7
    x1 x2 e cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry.readAbstractCurryFile._#lambda6._#lambda7" [show x1, show x2
    , show x3] info)
  _ -> failCons cd (consFail
    "AbstractCurry.readAbstractCurryFile._#lambda6._#lambda7" (showCons x3))

d_C_tryReadACYFile :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover
  -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
d_C_tryReadACYFile x1 cd cs = let x2 = Curry_Prelude.d_C_return
                                    Curry_Prelude.C_Nothing cd cs
  in Curry_Prelude.d_OP_gt_gt_eq (Curry_Directory.d_C_doesFileExist x1 cd cs)
  (d_OP_tryReadACYFile_dot_uscore_hash_lambda10 x2 x1) cd cs

d_OP_tryReadACYFile_dot_tryRead_dot_31 :: Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover
  -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
d_OP_tryReadACYFile_dot_tryRead_dot_31 x1 x2 x3 cd cs =
  Curry_Prelude.d_OP_gt_gt_eq (Curry_Prelude.d_C_readFile x3 cd cs)
  (d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8 x1 x2) cd cs

nd_OP_tryReadACYFile_dot_tryRead_dot_31 :: Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> IDSupply
  -> Cover -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe
  C_CurryProg)
nd_OP_tryReadACYFile_dot_tryRead_dot_31 x1 x2 x3 s cd cs = let s0 = s
  in s0 `seq` Curry_Prelude.nd_OP_gt_gt_eq (Curry_Prelude.d_C_readFile x3 cd cs)
  (wrapNX id (nd_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8 x1
  x2)) s0 cd cs

d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8 :: Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Cover
  -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8 x1 x2 x3 cd cs =
  let x4 = Curry_Prelude.d_C_apply (Curry_Prelude.d_C_break
        (Curry_Prelude.d_C_flip (acceptCs id Curry_Prelude.d_OP_eq_eq)
        (Curry_Prelude.C_Char '\n'#)) cd cs) x3 cd cs
      x5 = d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP5_hash_line1
        x4 cd cs
      x6 = d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP6_hash_lines
        x4 cd cs
  in d_C__case_4 x5 x6 x1 x2 (Curry_Prelude.d_OP_slash_eq x5
  (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "{- ")
  (Curry_Prelude.d_OP_plus_plus (d_C_version cd cs) (Curry_Prelude.toCurryString
  " -}") cd cs) cd cs) cd cs) cd cs

nd_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8 :: Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> IDSupply
  -> Cover -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe
  C_CurryProg)
nd_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8 x1 x2 x3 s cd cs =
  let s4 = s
  in s4 `seq` (let s2 = leftSupply s4
                   s3 = rightSupply s4
  in s2 `seq` (s3 `seq` (let x4 = let s1 = leftSupply s2
                                      s0 = rightSupply s2
                               in s1 `seq` (s0 `seq` Curry_Prelude.nd_C_apply
                               (Curry_Prelude.nd_C_break (wrapNX id
                               (Curry_Prelude.nd_C_flip (wrapDX (wrapDX id)
                               (acceptCs id Curry_Prelude.d_OP_eq_eq))
                               (Curry_Prelude.C_Char '\n'#))) s0 cd cs) x3 s1 cd
                               cs)
                             x5 = d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP5_hash_line1
                               x4 cd cs
                             x6 = d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP6_hash_lines
                               x4 cd cs
  in nd_C__case_4 x5 x6 x1 x2 (Curry_Prelude.d_OP_slash_eq x5
  (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "{- ")
  (Curry_Prelude.d_OP_plus_plus (d_C_version cd cs) (Curry_Prelude.toCurryString
  " -}") cd cs) cd cs) cd cs) s3 cd cs)))

d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP5_hash_line1 :: Curry_Prelude.OP_Tuple2
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List
  Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP5_hash_line1 x1 cd cs =
  case x1 of
  Curry_Prelude.OP_Tuple2 x2 x3 -> x2
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i
    (d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP5_hash_line1
    l cd cs)
    (d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP5_hash_line1
    r cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z ->
    d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP5_hash_line1
    z cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c
    (d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP5_hash_line1
    e cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "AbstractCurry.tryReadACYFile.tryRead.31._#lambda8._#selFP5#line1" [show x1]
    info)
  _ -> failCons cd (consFail
    "AbstractCurry.tryReadACYFile.tryRead.31._#lambda8._#selFP5#line1" (showCons
    x1))

d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP6_hash_lines :: Curry_Prelude.OP_Tuple2
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) (Curry_Prelude.OP_List
  Curry_Prelude.C_Char) -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP6_hash_lines x1 cd cs =
  case x1 of
  Curry_Prelude.OP_Tuple2 x2 x3 -> x3
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i
    (d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP6_hash_lines
    l cd cs)
    (d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP6_hash_lines
    r cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z ->
    d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP6_hash_lines
    z cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c
    (d_OP_tryReadACYFile_dot_tryRead_dot_31_dot_uscore_hash_lambda8_dot_uscore_hash_selFP6_hash_lines
    e cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "AbstractCurry.tryReadACYFile.tryRead.31._#lambda8._#selFP6#lines" [show x1]
    info)
  _ -> failCons cd (consFail
    "AbstractCurry.tryReadACYFile.tryRead.31._#lambda8._#selFP6#lines" (showCons
    x1))

d_OP_tryReadACYFile_dot_uscore_hash_lambda10 :: Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore
  -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
d_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 x3 cd cs = case x3 of
  Curry_Prelude.C_True -> d_OP_tryReadACYFile_dot_tryRead_dot_31 x1 x2 x2 cd cs
  Curry_Prelude.C_False -> let x4 = Curry_Distribution.d_C_inCurrySubdir x2 cd
                                 cs
    in Curry_Prelude.d_OP_gt_gt_eq (Curry_Directory.d_C_doesFileExist x4 cd cs)
    (d_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1 x2
    x4) cd cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i
    (d_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 l cd cs)
    (d_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z ->
    d_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c
    (d_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 e cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry.tryReadACYFile._#lambda10" [show x1, show x2, show x3] info)
  _ -> failCons cd (consFail "AbstractCurry.tryReadACYFile._#lambda10" (showCons
    x3))

nd_OP_tryReadACYFile_dot_uscore_hash_lambda10 :: Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> IDSupply -> Cover
  -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
nd_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 x3 s cd cs = case x3 of
  Curry_Prelude.C_True -> let s0 = s
    in s0 `seq` nd_OP_tryReadACYFile_dot_tryRead_dot_31 x1 x2 x2 s0 cd cs
  Curry_Prelude.C_False -> let s0 = s
    in s0 `seq` (let x4 = Curry_Distribution.d_C_inCurrySubdir x2 cd cs
    in Curry_Prelude.nd_OP_gt_gt_eq (Curry_Directory.d_C_doesFileExist x4 cd cs)
    (wrapNX id
    (nd_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1
    x2 x4)) s0 cd cs)
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i
    (nd_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 l s cd cs)
    (nd_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 r s cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z ->
    nd_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 z s cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c
    (nd_OP_tryReadACYFile_dot_uscore_hash_lambda10 x1 x2 e s cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry.tryReadACYFile._#lambda10" [show x1, show x2, show x3] info)
  _ -> failCons cd (consFail "AbstractCurry.tryReadACYFile._#lambda10" (showCons
    x3))

d_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 :: Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char
  -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg)
d_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1 x2 x3 x4 cd cs =
  case x4 of
  Curry_Prelude.C_True -> d_OP_tryReadACYFile_dot_tryRead_dot_31 x1 x2 x3 cd cs
  Curry_Prelude.C_False -> x1
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i
    (d_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1 x2
    x3 l cd cs)
    (d_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1 x2
    x3 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z ->
    d_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1 x2
    x3 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c
    (d_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1 x2
    x3 e cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry.tryReadACYFile._#lambda10._#lambda11" [show x1, show x2, show
    x3, show x4] info)
  _ -> failCons cd (consFail
    "AbstractCurry.tryReadACYFile._#lambda10._#lambda11" (showCons x4))

nd_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 :: Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char
  -> Curry_Prelude.C_Bool -> IDSupply -> Cover -> ConstStore
  -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
nd_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1 x2 x3 x4 s cd cs =
  case x4 of
  Curry_Prelude.C_True -> let s0 = s
    in s0 `seq` nd_OP_tryReadACYFile_dot_tryRead_dot_31 x1 x2 x3 s0 cd cs
  Curry_Prelude.C_False -> x1
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i
    (nd_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1
    x2 x3 l s cd cs)
    (nd_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1
    x2 x3 r s cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z ->
    nd_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1 x2
    x3 z s cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c
    (nd_OP_tryReadACYFile_dot_uscore_hash_lambda10_dot_uscore_hash_lambda11 x1
    x2 x3 e s cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry.tryReadACYFile._#lambda10._#lambda11" [show x1, show x2, show
    x3, show x4] info)
  _ -> failCons cd (consFail
    "AbstractCurry.tryReadACYFile._#lambda10._#lambda11" (showCons x4))

d_C_writeAbstractCurryFile :: Curry_Prelude.OP_List Curry_Prelude.C_Char
  -> C_CurryProg -> Cover -> ConstStore -> Curry_Prelude.C_IO
  Curry_Prelude.OP_Unit
d_C_writeAbstractCurryFile x1 x2 cd cs = Curry_Prelude.d_C_writeFile x1
  (Curry_ReadShowTerm.d_C_showTerm x2 cd cs) cd cs

d_C__case_4 :: Curry_Prelude.OP_List Curry_Prelude.C_Char
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore
  -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
d_C__case_4 x5 x6 x1 x2 x7 cd cs = case x7 of
  Curry_Prelude.C_True -> Curry_Prelude.d_OP_dollar Curry_Prelude.d_C_error
    (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString
    "AbstractCurry: incompatible file found: ") x2 cd cs) cd cs
  Curry_Prelude.C_False -> d_C__case_3 x6 x1
    (Curry_ReadShowTerm.d_C_readsUnqualifiedTerm (Curry_Prelude.OP_Cons
    (Curry_Prelude.toCurryString "AbstractCurry") (Curry_Prelude.OP_Cons
    (Curry_Prelude.toCurryString "Prelude") Curry_Prelude.OP_List)) x6 cd cs) cd
    cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C__case_4 x5 x6 x1 x2 l
    cd cs) (d_C__case_4 x5 x6 x1 x2 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C__case_4 x5 x6
    x1 x2 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C__case_4 x5 x6 x1 x2 e
    cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry._case_4" [show x5, show x6, show x1, show x2, show x7] info)
  _ -> failCons cd (consFail "AbstractCurry._case_4" (showCons x7))

nd_C__case_4 :: Curry_Prelude.OP_List Curry_Prelude.C_Char
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> IDSupply -> Cover
  -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
nd_C__case_4 x5 x6 x1 x2 x7 s cd cs = case x7 of
  Curry_Prelude.C_True -> let s0 = s
    in s0 `seq` Curry_Prelude.nd_OP_dollar (wrapDX id Curry_Prelude.d_C_error)
    (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString
    "AbstractCurry: incompatible file found: ") x2 cd cs) s0 cd cs
  Curry_Prelude.C_False -> let s0 = s
    in s0 `seq` nd_C__case_3 x6 x1 (Curry_ReadShowTerm.d_C_readsUnqualifiedTerm
    (Curry_Prelude.OP_Cons (Curry_Prelude.toCurryString "AbstractCurry")
    (Curry_Prelude.OP_Cons (Curry_Prelude.toCurryString "Prelude")
    Curry_Prelude.OP_List)) x6 cd cs) s0 cd cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (nd_C__case_4 x5 x6 x1 x2 l
    s cd cs) (nd_C__case_4 x5 x6 x1 x2 r s cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> nd_C__case_4 x5
    x6 x1 x2 z s cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (nd_C__case_4 x5 x6 x1 x2 e
    s cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry._case_4" [show x5, show x6, show x1, show x2, show x7] info)
  _ -> failCons cd (consFail "AbstractCurry._case_4" (showCons x7))

d_C__case_3 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  (Curry_Prelude.OP_Tuple2 C_CurryProg (Curry_Prelude.OP_List
  Curry_Prelude.C_Char)) -> Cover -> ConstStore -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg)
d_C__case_3 x6 x1 x9 cd cs = case x9 of
  Curry_Prelude.OP_List -> x1
  Curry_Prelude.OP_Cons x7 x8 -> d_C__case_2 x8 x1 x7 cd cs
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C__case_3 x6 x1 l cd cs)
    (d_C__case_3 x6 x1 r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C__case_3 x6
    x1 z cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C__case_3 x6 x1 e cd $!
    addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "AbstractCurry._case_3" [show x6, show x1, show x9] info)
  _ -> failCons cd (consFail "AbstractCurry._case_3" (showCons x9))

nd_C__case_3 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_List
  (Curry_Prelude.OP_Tuple2 C_CurryProg (Curry_Prelude.OP_List
  Curry_Prelude.C_Char)) -> IDSupply -> Cover -> ConstStore
  -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
nd_C__case_3 x6 x1 x9 s cd cs = case x9 of
  Curry_Prelude.OP_List -> x1
  Curry_Prelude.OP_Cons x7 x8 -> let s0 = s
    in s0 `seq` nd_C__case_2 x8 x1 x7 s0 cd cs
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (nd_C__case_3 x6 x1 l s cd
    cs) (nd_C__case_3 x6 x1 r s cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> nd_C__case_3 x6
    x1 z s cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (nd_C__case_3 x6 x1 e s
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "AbstractCurry._case_3" [show x6, show x1, show x9] info)
  _ -> failCons cd (consFail "AbstractCurry._case_3" (showCons x9))

d_C__case_2 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_CurryProg
  (Curry_Prelude.OP_List Curry_Prelude.C_Char)) -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_Tuple2 C_CurryProg
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover -> ConstStore
  -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
d_C__case_2 x8 x1 x7 cd cs = case x7 of
  Curry_Prelude.OP_Tuple2 x9 x10 -> d_C__case_1 x1 x10 x9 x8 cd cs
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i (d_C__case_2 x8 x1 l cd
    cs) (d_C__case_2 x8 x1 r cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z -> d_C__case_2 x8
    x1 z cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c (d_C__case_2 x8 x1 e
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "AbstractCurry._case_2" [show x8, show x1, show x7] info)
  _ -> failCons cd (consFail "AbstractCurry._case_2" (showCons x7))

nd_C__case_2 :: Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_CurryProg
  (Curry_Prelude.OP_List Curry_Prelude.C_Char)) -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> Curry_Prelude.OP_Tuple2 C_CurryProg
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> IDSupply -> Cover
  -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
nd_C__case_2 x8 x1 x7 s cd cs = case x7 of
  Curry_Prelude.OP_Tuple2 x9 x10 -> let s0 = s
    in s0 `seq` nd_C__case_1 x1 x10 x9 x8 s0 cd cs
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i (nd_C__case_2 x8 x1 l s
    cd cs) (nd_C__case_2 x8 x1 r s cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z -> nd_C__case_2
    x8 x1 z s cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c (nd_C__case_2 x8 x1 e s
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "AbstractCurry._case_2" [show x8, show x1, show x7] info)
  _ -> failCons cd (consFail "AbstractCurry._case_2" (showCons x7))

d_C__case_1 :: Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> C_CurryProg
  -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_CurryProg
  (Curry_Prelude.OP_List Curry_Prelude.C_Char)) -> Cover -> ConstStore
  -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
d_C__case_1 x1 x10 x9 x8 cd cs = case x8 of
  Curry_Prelude.OP_List -> d_C__case_0 x10 x1 x9 (Curry_Prelude.d_C_apply
    (Curry_Prelude.d_C_all Curry_Char.d_C_isSpace cd cs) x10 cd cs) cd cs
  Curry_Prelude.OP_Cons x11 x12 -> x1
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C__case_1 x1 x10 x9 l cd
    cs) (d_C__case_1 x1 x10 x9 r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C__case_1 x1
    x10 x9 z cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C__case_1 x1 x10 x9 e
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "AbstractCurry._case_1" [show x1, show x10, show x9, show x8] info)
  _ -> failCons cd (consFail "AbstractCurry._case_1" (showCons x8))

nd_C__case_1 :: Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> C_CurryProg
  -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 C_CurryProg
  (Curry_Prelude.OP_List Curry_Prelude.C_Char)) -> IDSupply -> Cover
  -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe C_CurryProg)
nd_C__case_1 x1 x10 x9 x8 s cd cs = case x8 of
  Curry_Prelude.OP_List -> let s4 = s
    in s4 `seq` (let s3 = leftSupply s4
                     s2 = rightSupply s4
    in s3 `seq` (s2 `seq` nd_C__case_0 x10 x1 x9 (let s1 = leftSupply s2
                                                      s0 = rightSupply s2
    in s1 `seq` (s0 `seq` Curry_Prelude.nd_C_apply (Curry_Prelude.nd_C_all
    (wrapDX id Curry_Char.d_C_isSpace) s0 cd cs) x10 s1 cd cs)) s3 cd cs))
  Curry_Prelude.OP_Cons x11 x12 -> x1
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (nd_C__case_1 x1 x10 x9 l s
    cd cs) (nd_C__case_1 x1 x10 x9 r s cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> nd_C__case_1 x1
    x10 x9 z s cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (nd_C__case_1 x1 x10 x9 e s
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "AbstractCurry._case_1" [show x1, show x10, show x9, show x8] info)
  _ -> failCons cd (consFail "AbstractCurry._case_1" (showCons x8))

d_C__case_0 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> C_CurryProg -> Curry_Prelude.C_Bool
  -> Cover -> ConstStore -> Curry_Prelude.C_IO (Curry_Prelude.C_Maybe
  C_CurryProg)
d_C__case_0 x10 x1 x9 x11 cd cs = case x11 of
  Curry_Prelude.C_True -> Curry_Prelude.d_OP_dollar Curry_Prelude.d_C_return
    (Curry_Prelude.C_Just x9) cd cs
  Curry_Prelude.C_False -> x1
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C__case_0 x10 x1 x9 l cd
    cs) (d_C__case_0 x10 x1 x9 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C__case_0 x10
    x1 x9 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C__case_0 x10 x1 x9 e
    cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry._case_0" [show x10, show x1, show x9, show x11] info)
  _ -> failCons cd (consFail "AbstractCurry._case_0" (showCons x11))

nd_C__case_0 :: Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg) -> C_CurryProg -> Curry_Prelude.C_Bool
  -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.C_IO
  (Curry_Prelude.C_Maybe C_CurryProg)
nd_C__case_0 x10 x1 x9 x11 s cd cs = case x11 of
  Curry_Prelude.C_True -> let s0 = s
    in s0 `seq` Curry_Prelude.nd_OP_dollar (wrapDX id Curry_Prelude.d_C_return)
    (Curry_Prelude.C_Just x9) s0 cd cs
  Curry_Prelude.C_False -> x1
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (nd_C__case_0 x10 x1 x9 l s
    cd cs) (nd_C__case_0 x10 x1 x9 r s cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> nd_C__case_0 x10
    x1 x9 z s cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (nd_C__case_0 x10 x1 x9 e s
    cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry._case_0" [show x10, show x1, show x9, show x11] info)
  _ -> failCons cd (consFail "AbstractCurry._case_0" (showCons x11))

d_C__case_5 ::(Curry_Prelude.Curry t0) => Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Curry_Prelude.OP_List Curry_Prelude.C_Char
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover
  -> ConstStore -> Curry_Prelude.C_IO t0
d_C__case_5 x4 x1 x5 x6 cd cs = case x6 of
  Curry_Prelude.C_True -> Curry_Prelude.d_C_return
    (Curry_ReadShowTerm.d_C_readUnqualifiedTerm (Curry_Prelude.OP_Cons
    (Curry_Prelude.toCurryString "AbstractCurry") (Curry_Prelude.OP_Cons
    (Curry_Prelude.toCurryString "Prelude") Curry_Prelude.OP_List)) x5 cd cs) cd
    cs
  Curry_Prelude.C_False -> Curry_Prelude.d_OP_dollar Curry_Prelude.d_C_error
    (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString
    "AbstractCurry: incompatible file found: ") x1 cd cs) cd cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C__case_5 x4 x1 x5 l cd
    cs) (d_C__case_5 x4 x1 x5 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C__case_5 x4 x1
    x5 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C__case_5 x4 x1 x5 e
    cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "AbstractCurry._case_5" [show x4, show x1, show x5, show x6] info)
  _ -> failCons cd (consFail "AbstractCurry._case_5" (showCons x6))

